###################################################################
package BU::DB;
###################################################################

require 5.004;
require Exporter;
use Carp;
use DBI;

use BU::Init;
use BU::Logger;

@ISA = qw(Exporter);
@EXPORT = qw(dbConnect dbDoQuery dbQuote dbPing dbGetInsertId dbCorrectDBName);

my $dbh;


sub dbConnect($ $ $; $)
{
	my ($dbLogin, $dbPasswd, $dbName, $params) = @_;

	unless (defined($dbLogin))	{ print_stderr("Unable to connect to database: login is undefined.\n"); return ''; }
	unless (defined($dbPasswd))	{ print_stderr("Unable to connect to database: password is undefined.\n"); return '';}
	unless (defined($dbName))	{ print_stderr("Unable to connect to database: database name is undefined.\n"); return '';}

	my $driver = 'mysql';

	my $socket_file = &dbLocateMysqlSocket(defined($params) ? $$params{'socket_file'} : undef);
	unless ($socket_file) {
		print_stderr("Unable to find file mysql.sock, possible reason: mysql service is not started.\n");
		warn_log "%.Unable to find file mysql.sock, possible reason: mysql service is not started.\n";
		return '';
	}

	my $dataSource  = "DBI:$driver:host=localhost;database=$dbName;mysql_socket=$socket_file";

	unless ($dbh = DBI->connect($dataSource, $dbLogin, $dbPasswd, \%attr)) {
		warn_log "%.Unable to connect to $driver\n";
		return '';
	}

	return 1;
}

sub dbDoQuery($)
{
	my ($query_str) = @_;

	print_stdout ("query is: ". $query_str. "\n") if ($debug>0);

	my $query = $dbh->prepare($query_str) || 
		warn_log "ERROR on exec mysql prepare: $query_str\n";
	unless ($query->execute()) {
		warn_log "ERROR on exec mysql query  : $query_str\n";
		return '';
	}

	return $query;
}

sub dbLocateMysqlSocket ($)
{
	my ($socket_path) = @_;

	return "$socket_path" if ( -e "$socket_path") ;

	$socket_path='/tmp/mysql.sock';	# default path
	return "$socket_path" if ( -e "$socket_path") ;

	$socket_path='/var/lib/mysql/mysql.sock';	# default path
	return "$socket_path" if ( -e "$socket_path") ;

	if ( (-e $const{MYSQL_CONF}) and (open FD, $const{MYSQL_CONF}) ) {
		while (<FD>) {
			chomp;
			if (m/^\s*socket\s*=\s*([\S]+)\s*$/) {
				$socket_path=$1; 
				last;
			}
		}
		close FD;
		return "$socket_path" if ( -e "$socket_path");
	} else {
		return '';
	}
}


sub dbQuote($)
{
	if ($dbh) {
		return $dbh->quote(@_[0]);
	}

	return @_[0];
}


sub dbPing()
{
	return $dbh->ping();
}


sub dbGetInsertId()
{
	return $dbh->{'mysql_insertid'};
}


sub dbCorrectDBName($)
{
	my ($dbName) = @_;

	$dbName =~ s/\\_/_/g;
	$dbName =~ s/_/\\_/g;

	return dbQuote($dbName);
}


return 1;

