package BU::PSA::MIME::Add2Stream;

require 5.004;
require Exporter;

use Digest::MD5 qw(md5_hex);

use BU::Logger;
use BU::MIME::Container;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::DB;
use BU::PSA::Const;

use IPC::SysV qw(IPC_PRIVATE S_IRWXU IPC_CREAT IPC_EXCL SEM_UNDO);
use BU::Semaphore;
use XML::DOM;

@ISA = qw(Exporter);
@EXPORT = qw(
pipe_util_execstr
addTextCertKey2Stream
addFile2Stream
addDir2Stream
addMysql2Stream
addPostgres2Stream
addLogo2Stream
addDomain2Stream
addMailbox2Stream
addAttach2Stream
addAdminCert2Stream
addAdminRootChainCert2Stream
addDomainRootChainCert2Stream
getPathToAttachRepos
addFakeNode2Stream
addCrontab2Stream
addSubDomain2Stream
);

#
# Make STREAM.
#

sub pipe_util_execstr
{
	my ($subject, $ignore_errors, $uid, $domain_for_turning_off, @commands) = @_;

	$uid = ($uid ne '') ? $uid : 0;

	my $util = $conf{PIPE_UTIL}
			. ' ' .semGetKey()
			. ' "' . getProcessingLogFileName() . '"'
			. " \"$subject\""
			. " \"$uid\""
			. " \"$ignore_errors\" "
			. " \"$domain_for_turning_off\"";

	my $args = '';
	foreach my $arg (@commands) {
		next unless $arg;
		my $encoded = url_encode($arg);
		$args .= ' ' . $encoded;
	}

	return $util . $args . ' |';
}


#
# Common funcs.:
#
sub addFile2Stream {
	my ( $node, $name, $path, $type, $ignore ) = @_;

	unless( -r $path ) {
		return 0 if( $ignore );
		printf_stderr("Unable to locate file %s: %s\n", $path, $!);
		return 0;
	}

	# Generate unique cid from file directory/name
	my $cid=$node->getNodeName.":$path:$name";
	$cid =~ s/ /_/g;
	$cid =~ s/\//:/g;
	$cid =~ s/::/:/g;
	$cid = md5_hex($cid);

	$node->setAttribute('src', "cid:$cid");
	makePartOfContainer(cid =>$cid, name => $name, path => $path, 
		type => $type);
	return 0;
}

#############################################
#
# Dumping contents of system user's crontab
#
#############################################
sub addCrontab2Stream {
	my ( $node, $user ) = @_;

	my $data;

	# check that crontab exists for system user
	if (not open(CTM, $conf{UTILS_DIR}."/crontabmng get $user |")) {
		warn_log "Can't access crontab state for user $user";
		return 0;
	}

	# get a first line - if it's empty - no crontab exists
	my $line = <CTM>;
	my $line_copy = $line;

	$line_copy =~ s/^\s+//;
	$line_copy =~ s/\s+$//;

	# no crontab for given user
	if (length($line_copy) == 0) {
		close CTM;
		return 0;
	}

	print_stderr("dumping crontab of $user\n");

	# read the rest of crontab, skipping first lines with
	# crontab-generated comment
	while (	$line =~ /^# DO NOT EDIT THIS FILE - edit the master and reinstall./ || 
			$line =~ /^# \(.*\)/) {
		$line = <CTM>;
	}

	do {
		# skip last line containing return code of crontabmng utility
		if (not $line =~ /^0$/)
			{ $data .= $line; }

	} while ($line = <CTM>);

	close CTM;

	# remove last empty line(s) added by crontabmng
	$data =~ s/(\s+$)+/\n/;

	# generate unique cid
	my $cid = md5_hex("LinkinParkRulez crontab$user");

	$node->setAttribute('src', "cid:$cid");

	makePartOfContainer( cid => $cid, type => 'image/x-crontab', data => $data);

	return 0;
}
 
sub addDir2Stream($ $ $ $; $) {
	my ( $node, $uid, $name , $dpath, $path ) = @_;

	unless( -d "$dpath/$name" ) {
		warn_log "Couldn't open(dir): %s. Reason: %!.\n","$dpath/$name\n";
		return 0;
	}

	my $gzip_key = $flags{DISABLE_INTERNAL_GZIP} ? '' : 'z';
	my $tar_ignore_failed_read = ($uid || $flags{TAR_IGNORE_FAILED_READ}) ? '--ignore-failed-read' : '';
	my $ignore_errors = 0;

	unless ($path) {
		my $tar_output_redirect_str = $uid ? ' 2>/dev/null' : '';

		$path = pipe_util_execstr(
			"dumping directory '$dpath/$name'",
			$ignore_errors, $uid,
			'',
			$const{tar} . " -c$gzip_key $tar_ignore_failed_read -f - -C '$dpath' '$name' $tar_output_redirect_str"
		);
	}

	my $cid=$node->getNodeName.":$dpath:$name:$uid";
	$cid =~ s/ /_/g;
	$cid = md5_hex($cid);

	$node->setAttribute('src', "cid:$cid");

	makePartOfContainer( cid =>$cid, name => $name, path => $path, 
		type => 'application/x-dir', gzip => ($flags{DISABLE_INTERNAL_GZIP} ?  '' : 1) );

	return 1;
}

#
# Special funcs.:
#
sub addMysql2Stream {
	my ($node, $name, $fl) = @_;

	return 0 unless ($$fl{'force'} or ($flags{'BACKUP_TYPE'} eq 'FULL'));

	my ($username, $password) = ($conf{ADMIN_USER},$conf{ADMIN_PASSWD});

	my $gzip_str = $flags{DISABLE_INTERNAL_GZIP} ? '' : '"gzip"';
	my $ignore_errors = 0;

	$password = shellArgQuote($password);

#	my $path =  $conf{PIPE_UTIL}
#			. ' ' . semGetKey()
#			. ' "' . getProcessingLogFileName() . '"'
#			. " \"dumping mysql database $name\""
#			. " $ignore_errors "
#			. " \"\" "
#			. ' "' . $conf{MYSQL_BIN_D}."/mysqldump -u$username -p$password"
#				   . ' --quote-names '.dbQuote($name) . '"'
#			. " $gzip_str |";

	my $path = pipe_util_execstr(
			"dumping mysql database $name",
			$ignore_errors, 0,
			'',
			$conf{MYSQL_BIN_D}."/mysqldump -u$username -p$password --quote-names " . shellArgQuote($name),
			$gzip_str
	);

	my $cid=$node->getNodeName.":$name:MySQL:LimpBizkit";
	$cid =~ s/ /_/g;
	$cid = md5_hex($cid);

	$node->setAttribute('src', "cid:$cid");

	makePartOfContainer( cid =>$cid, name => $name, path => $path, 
						type => 'application/x-mysql-dump',
						gzip => ($flags{DISABLE_INTERNAL_GZIP} ?  '' : 1) );

	return 1;
}


sub addFakeNode2Stream
{
	my ($path) = @_;

	my $ignore_errors = 0;

	my $cid = 'fake_node';
	my $name = $cid;

	makePartOfContainer(cid => $cid, name => $name, path => $path, 
						type => 'application/x-fake-node',);

	return 1;
}

sub addPostgres2Stream {
	my ($node, $name, $fl) = @_;

	return 0 unless ($$fl{'force'} or ($flags{'BACKUP_TYPE'} eq 'FULL'));

	my $gzip_str = $flags{DISABLE_INTERNAL_GZIP} ? '' : '"gzip"';
	my $ignore_errors = 0;

	$username = shellArgQuote($conf{PG_ADMIN_LOGIN});
	$password = shellArgQuote($conf{PG_ADMIN_PASSWORD});

	my $path = pipe_util_execstr(
			"dumping postgres database $name",
			$ignore_errors, 0,
			'',
			"export PGUSER=$username; export PGPASSWORD=$password; " . $conf{PGSQL_BIN_D} . "/pg_dump -Fc -b -o " . shellArgQuote($name),
			$gzip_str
	);


	my $cid=$node->getNodeName.":$name:PostgreSQL:LimpBizkit";
	$cid =~ s/ /_/g;
	$cid = md5_hex($cid);

	$node->setAttribute('src', "cid:$cid");

	makePartOfContainer( cid =>$cid, name => $name, path => $path, 
						type => 'application/x-pgsql-dump',
						gzip => ($flags{DISABLE_INTERNAL_GZIP} ?  '' : 1) );

	return 1;
}


sub addLogo2Stream {
	my ( $node, $name, $alias ) = @_;
	my $path =  $conf{LOGOS_REPOSITORY_D} . "/$name";

	return addFile2Stream($node, ($alias ? $alias : $name), $path, 'image/x-logo');
}


sub addDomain2Stream {
	my ($node, $uid, $name, $fl) = @_;

	return 0 unless ($$fl{'force'} or ($flags{'BACKUP_TYPE'} eq 'FULL'));

	my $gzip_key = $flags{DISABLE_INTERNAL_GZIP} ? '' : 'z';
	my $tar_ignore_failed_read = ($uid || $flags{TAR_IGNORE_FAILED_READ}) ? '--ignore-failed-read' : '';
	my $ignore_errors = 0; 
	my $dpath = $conf{HTTPD_VHOSTS_D} . "/$name";
	my $exclude_logs = $flags{'DO_NOT_DUMP_LOGS'} ? "--exclude='./statistics/logs'" : '';

	my $dirs_for_backup = '';
	my $exclude_opts = "--exclude='./web_users' --exclude='./tmp' --exclude='./etc' --exclude='./lib' --exclude='./bin' --exclude='./usr' --exclude='./subdomains' --exclude='./conf/httpd.include*' $exclude_logs";
	if ($uid) {
#		$dirs_for_backup = 'anon_ftp cgi-bin httpdocs httpsdocs private';
		$dirs_for_backup = '.';
		$exclude_opts .= " --exclude='./conf' --exclude='./error_docs' --exclude='./pd' --exclude='./statistics'";
		$tar_output_redirect_str = ' 2>/dev/null';
	}else {
		$dirs_for_backup = 'conf error_docs pd statistics';
		if (-d "$dpath/vault_scripts") {
			$dirs_for_backup .= ' vault_scripts';
		}
		$tar_output_redirect_str = '';
	}

	my $comment = $uid ? "dumping hosting user data for domain '$dpath'" : "dumping hosting system data for domain '$dpath'";

	my $path = pipe_util_execstr(
			$comment,
			$ignore_errors, $uid,
			(($flags{turn_domain_off_before_backup} && ($name !~ /^(\.skel|default)$/)) ? $name : ''),
			$const{tar} . " -c$gzip_key $tar_ignore_failed_read $exclude_opts -f - -C '$dpath' $dirs_for_backup $tar_output_redirect_str"
	);

	return addDir2Stream($node, $uid, '.', $dpath, $path);	
}


sub addSubDomain2Stream {
	my ($node, $uid, $domain_name, $sd_name, $fl) = @_;

	return 0 unless ($$fl{'force'} or ($flags{'BACKUP_TYPE'} eq 'FULL'));

	my $gzip_key = $flags{DISABLE_INTERNAL_GZIP} ? '' : 'z';
	my $tar_ignore_failed_read = ($uid || $flags{TAR_IGNORE_FAILED_READ}) ? '--ignore-failed-read' : '';
	my $ignore_errors = 1;
	my $dpath = $conf{HTTPD_VHOSTS_D} . "/$domain_name/subdomains/$sd_name";
	my $exclude_logs = $flags{'DO_NOT_DUMP_LOGS'} ? "--exclude='logs'" : '';

	my $dirs_for_backup = '';
	my $exclude_opts = "$exclude_logs";
	if ($uid) {
		$dirs_for_backup = '.';
		$exclude_opts .= " --exclude='./conf'";
		$tar_output_redirect_str = ' 2>/dev/null';
	}else {
		$dirs_for_backup = 'conf';
		if (-d "$dpath/vault_scripts") {
			$dirs_for_backup .= ' vault_scripts';
		}
		$tar_output_redirect_str = '';
	}

	my $tar_output_redirect_str = $uid ? ' 2>/dev/null' : '';
	my $path = pipe_util_execstr(
			"dumping subdomain hosting '$dpath'",
			$ignore_errors, $uid,
			(($flags{turn_domain_off_before_backup} && ($domain_name !~ /^(\.skel|default)$/)) ? $domain_name : ''),
			$const{tar} . " -c$gzip_key $tar_ignore_failed_read $exclude_opts -f - -C '$dpath' $dirs_for_backup $tar_output_redirect_str"
	);

	return addDir2Stream($node, $uid, '.', $dpath, $path);	
}


sub  addMailbox2Stream {
	my ($node, $name, $fl) = @_;

	return 0 unless ($$fl{'force'} or ($flags{'BACKUP_TYPE'} eq 'FULL'));

	# exclude mailname from tar archive
	return addDir2Stream($node, 0, '.', $conf{QMAIL_MAILNAMES_D} . "/$name");
}


sub  addAttach2Stream {
	my ( $node, $name ) = @_;

	my $mail=$node->getParentNode->getParentNode;
	my $path = &getPathToAttachRepos ($mail) . "/$name";

	return addFile2Stream($node, $name, $path, 'image/x-attachment');
}


sub addDomainRootChainCert2Stream {
	my ( $node, $domainName ) = @_;
	my $name = 'rootchain.pem';
	my $path = $conf{HTTPD_VHOSTS_D} . "/$domainName/cert/$name";

	return '' unless (existFile($path));

	return addFile2Stream($node, $name, $path, 'image/x-certificate');
}


sub addAdminRootChainCert2Stream {
	my ( $node ) = @_;
	my $name = 'rootchain.pem';
	my $path = $conf{PRODUCT_ROOT_D} . "/admin/conf/$name";

	if (existFile($path)) {
		return addFile2Stream($node, $name, $path, 'image/x-certificate');
	}

	return 1;
}


sub  addAdminCert2Stream {
	my ( $node ) = @_;
	my $name = 'httpsd.pem';
	my $path = $conf{PRODUCT_ROOT_D} . "/admin/conf/$name";
	
	return addFile2Stream($node, $name, $path, 'image/x-certificate');
}


sub  addTextCertKey2Stream {
	my ( $node, $cert_text ) = @_;

	return 1 unless $cert_text;

	my $cid = md5_hex($cert_text . md5_hex(rand()));
	$node->setAttribute('src', "cid:$cid");

	makePartOfContainer( cid => $cid, type => 'image/x-certificate-key', data => $cert_text);

	return 1;
}


# additional subroutines

sub getPathToAttachRepos {
	my ($node) = @_;	
	
	my $user=($node->getElementsByTagName('login'))[0]->getAttribute('name');
	my $dom=$node->getParentNode->getAttribute('name');
#	print_stdout ("User = $user, Domain = $dom\n");	

	my $path = $conf{QMAIL_MAILNAMES_D}."/$dom/$user".'/@attachments';
	return $path;
}

1;

__END__

=head1 NAME

PSA::Add2Stream - The PSA module for ... .

=head1 SYNOPSIS

A short usage summary for programs and functions, which may someday be deemed mandatory. 

=head1 DESCRIPTION

Long drawn out discussion of the program. It's a good idea to break this up into subsections using the =head2 directives, like 

=head2 A Sample Subection

=head2 Yet Another Sample Subection

=head1 OPTIONS

Some people make this separate from the description. 

=head1 RETURN VALUE

What the program or function returns if successful. 

=head1 ERRORS

Exceptions, return codes, exit stati, and errno settings. 

=head1 EXAMPLES

Give some example uses of the program. 

=head1 ENVIRONMENT

Envariables this program might care about. 

=head1 FILES

All files used by the program. You should probably use the F<> for these. 

=head1 SEE ALSO

Other man pages to check out, like man(1), man(7), makewhatis(8), or catman(8). 

=head1 NOTES

Miscellaneous commentary. 

=head1 CAVEATS

Things to take special care with; sometimes called WARNINGS. 

=cut DIAGNOSTICS

All possible messages the program can print out--and what they mean. 

=head1 BUGS

