###################################################################
package BU::PSA::Objects::AnonFTP;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::CommonFunc;
use BU::FileFunc;
use BU::PSA::DB;
use BU::PSA::Logger;

use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init($ $)
{
	my $self = shift @_;
	my ($domainId, $domainName) = @_;

	my $fields = $self->{dbFields};

	defined($domainId) and $fields->{dom_id} = $domainId or croak "Argument 'domainId' is undefined.";
	defined($domainName) and $self->{domainName} = $domainName or croak "Argument 'domainName' is undefined.";

	$self->{dbFieldsNames} = { 'dom_id' => 1,
								'max_conn' => '', 'bandwidth' => '',
								'incoming' => '', 'status' => '',
								'incoming_readable' => '', 'incoming_subdirs' => '',
								'quota' => '',
								'display_login' => '',
								'login_text' => '',
							};

	# set values by default
	$fields->{max_conn} = 0;
	$fields->{bandwidth} = 0;
	$fields->{status} = 'false';
	$fields->{incoming} = 'false';

	$self->{incomingDir} = $conf{HTTPD_VHOSTS_D} . '/' . $self->{domainName}
							. '/anon_ftp/incoming';

	########################################################################
	#
	#	Hack for compatibility with Plesk < 6.0.2 In those versions quota 
	#	file was placed directly in incoming directory. In the later
	#	versions quota file was moved into incoming/quotadir subdirectory.
	#
	########################################################################

	if (ver2int($conf{PSA_VERSION}, 1) < ver2int("6.0.2", 1)) {

		############################
		#	Version prior to 6.0.2
		############################
		$self->{quotaFile} = $self->{incomingDir} . '/.quota';

	} else {

		#########################################
		#	Version equal or greater then 6.0.2
		#########################################
		$self->{quotaFile} = $self->{incomingDir} . '/quotadir/.quota';
	}

	$self->{incomingDirSize} = '';
	$self->getIncomingDirSize();

	$self->{dbTable} = 'anon_ftp';
	$self->{Changeable} = 1;

	$self->SUPER::init(@_);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'dom_id=' . $fields->{dom_id};
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return $self->dbGetIdCondition();
}


sub doCheckExists
{
	return '';
}


sub doCreate
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	$self->resetError();

#	unless ($self->{LockSysChanges}) {
		$self->saveQuota();
#	}

	return 1;
}


sub getIP()
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	$self->resetError();

	my $query = dbDoQuery('SELECT ip_address_id FROM hosting WHERE dom_id=' . $fields->{dom_id}); 

	unless ($query) {
		$self->setError("ERROR: Failed to retrieve anonymous FTP information from database.\n");
		return '';
	}

	my ($ip_address_id) = $query->fetchrow_array;

	my $ip = getIPAddressById($ip_address_id);

	return $ip;
}


sub putConf()
{
	my ($self) = @_;

	my $ret = undef;

	if ($self->{domainName}) {
		$ret = util_exec('ftpmng', '--reconfigure', '--vhost-name=' . $self->{domainName});
	} else {
		$ret = util_exec('ftpmng', '--reconfigure-all');
	}

	unless ($ret) {
		$self->setError('Unable to save protfd conf file: ftpmng util execution error' . 
						(util_exec_error() ? ': ' . util_exec_errormsg() : '.'));
	}

	return $ret;
}


sub saveQuota()
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	if ($fields->{quota} eq '')
	{
		$self->setError('FTP quota is not set.');
		return '';
	}

	my $temp_file = tempFileName($const{PRODUCT_NAME} . 'ftp.quota');

	unless (open FD2, ">$temp_file") {
		$self->setError('unable to open temporary file.');
		return '';
	}

	my $quota_str = sprintf("%f %f\n", $fields->{quota}, $self->getIncomingDirSize());
	print FD2 $quota_str;
	close FD2 || warn_log "ERROR occured during temporary FTP quota file %s writing: %!\n", $self->{quotaFile}."\n";

#	unless ($self->utilExec('copy_file', $temp_file, $self->{quotaFile})) {
	if (system(sprintf('cp -f %s %s 2>&1 >/dev/null', $temp_file, $self->{quotaFile})) != 0) {
		$self->setError('unable to save quota value into the file.');
		unlink($temp_file);
		return '';
	}

	unless($self->utilExec('ch_owner', 'psaftp:psacln', $self->{quotaFile})) {
		$self->setError('unable to change the FTP quota file ownership.');
		unlink($temp_file);
		return '';
	}

	unless (chmod 0644, $self->{quotaFile}) {
		$self->setError('unable to change the FTP quota file permissions.');
		unlink($temp_file);
		return '';
	}

	unlink($temp_file);
	return 1;
}


sub dbStore
{
	my $self = shift @_;

	$self->SUPER::dbStore(@_);

	$self->putConf();
}


sub getIncomingDirSize()
{
	my ($self) = @_;

#	unless (existDir($self->{incomingDir})) {
#		return 0;
#	}

	my $dir        = $self->{incomingDir};
	my $subdir     = $dir;
	print_stdout ("Estimating directory $dir size") if $debug>1;
	&existDir($dir) or return 0;
	
	my $ls_options = '-laR'; # +size, +allmost all, +recursive
	my $dir_size   = 0;

	unless ( open FD, "ls $ls_options $dir |" ) {
		warn_log "Directory $dir listing request failed\n";
		return 0;
	}	
	while (<FD>) {
		if (m/^([d\-]{1})([\S]{9})\s+(\d+)\s+(\w+)\s+(\w+)\s+(\d+)\s+([\S\s]*\S)\s+(\S+)$/) {
			# Parsing: type (directory or not), permissions, links, user, group, size, timestamp, filename
#			print_stdout ("Parsing ==> $1 :: $2 :: $3 :: $4 :: $5 :: $6 :: $7 :: $8\n");
			my @parsed_columns = ($1, $2, $3, $4, $5, $6, $7, $8);
			my ($propT, $propS, $propN) = @parsed_columns[0, 5, 7]; 
			print_stdout ("Got record ==> type=$propT; name=$propN; size=$propS\n") if $debug>2;
			$dir_size+=int($propS) if ($propT ne 'd' && "$subdir/$propN" ne $self->{quotaFile});	# exclude dirs and '.quota' file
		} elsif (m/(\/\S*):/) {
			$subdir = $1;
			print_stdout ("New sub-directory==> : $subdir\n") if $debug>2;
		} else {
			chomp;
			print_stdout ("Skipped line: $_\n") if $debug>2;
		}
	}
	close FD;

	return $dir_size;
}

return 1;

