###################################################################
package BU::PSA::Objects::Client;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::DB;

use BU::PSA::Objects::ObjectWithParams;

@ISA = qw(BU::PSA::Objects::ObjectWithParams);


sub init
{
	my $self = shift @_;
	my ($login, $pname, $passwd, $passwd_type) = @_;

#print_stdout ("init Client\n");

	my $fields = $self->{dbFields};

	$fields->{login} = $login;
	$fields->{pname} = $pname;
	$fields->{passwd} = $passwd;

	$self->{dbFieldsNames} = {
								'login' => 1,
								'pname' => 1, 'cr_date' => 'DATE', 'status' => 0, 'cname' => 0,
								'phone' => 0, 'fax' => 0, 'email' => 0,
								'address' => 0, 'city' => 0, 'state' => 0, 'pcode' => 0,
								'country' => 0, 'locale' => 0,
								'limits_id' => 0, 'pool_id' => 0, 'params_id' => 0, 'perm_id' => 0,
								'logo_id' => 0, 'tmpl_id' => 0,
								'sapp_pool_id' => 0,
								'account_id' => 0
							};

	$$self{action_log_map} = {
					'login' => COMP_LOGIN_NAME,
					'pname' => COMP_CONTACT_NAME,
					'status' => COMP_STATUS,
					'cname' => COMP_COMPANY_NAME,
					'phone' => COMP_PHONE,
					'fax' => COMP_FAX,
					'email' => COMP_EMAIL,
					'address' => COMP_ADDRESS,
					'city' => COMP_CITY,
					'state' => COMP_STATE_PROVINCE,
					'pcode' => COMP_POSTAL_ZIP_CODE,
					'country' => COMP_COUNTRY,
					'locale' => COMP_LOCALE
				};

	$self->{dbTable} = 'clients';
	$self->{dbParamsTable} = 'cl_param';
	$self->{dbRefTableIdName} = 'cl_id';
	$self->{dbParamFieldName} = 'param';
	$self->{dbValueFieldName} = 'val';
	$self->{Changeable} = 1;

	$self->SUPER::init(@_);

	my $account_id = $self->getField('account_id');
	$self->{objAccount} = new BU::PSA::Objects::Account($account_id ? $account_id : 0);
	$self->{objAccount}->setField('type', $passwd_type);
	$self->{objAccount}->setField('password', $passwd);

	return $self;
}

sub Store
{
	my $self = shift @_;

	$self->resetError();

	$self->{objAccount}->Store();

	if ($self->{objAccount}->error()) {
		$self->setError($self->{objAccount}->getErrorMsg());
		return undef;
	}

	$self->setField('account_id', $self->{objAccount}->getDBId());
	$self->SUPER::Store(@_);
}

sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'login=' . dbQuote($fields->{login});
}


sub dbExistsCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'login=' . dbQuote($fields->{login}) . ' and pname=' . dbQuote($fields->{pname});
}


sub doCheckExists()
{
	return 1;
}


return 1;

