###################################################################
package BU::PSA::Objects::DSTomcat;
###################################################################

require 5.004;
require Exporter;
use Carp;
use Digest::MD5 qw(md5_hex);
use BU::PSA::Const;
use BU::PSA::DB;
use BU::PSA::Objects::ObjectWithParams;

@ISA = qw(BU::PSA::Objects::ObjectWithParams);


sub init
{
	my $self = shift @_;
	my ($dom_id, $dom_name, $id, $www_prefix, $coyote_connector_port, $warp_connector_port) = @_;

	my $fields = $self->{dbFields};

	$id = 0 unless $id;

	$fields->{dom_id} = $dom_id;
	$self->{domainName} = $dom_name;
	$self->{coyote_connector_port} = $coyote_connector_port;
	$self->{warp_connector_port} = $warp_connector_port;
	$fields->{dbId} = $id;

	$self->{dbFieldsNames} = {'dom_id' => 1, 'type' => 0, 'status' => 0, 'parameters_id' => 0};

	$self->{dbTable} = 'DomainServices';
	$self->{Changeable} = 1;

	$self->{dbParamsTable} = 'Parameters';
	$self->{dbRefTableIdName} = 'id';
	$self->{dbParamFieldName} = 'parameter';
	$self->{dbValueFieldName} = 'value';
	$self->{dbParamsIdName} = 'parameters_id';

	if ($id == 0) {
		$self->{manager_login} = md5_hex(rand());
		$self->{manager_passwd} = md5_hex(rand());
		$self->setField('status', STATUS_STATUS);
		$self->setField('type', 'tomcat');
		$self->setField('dom_id', $dom_id);
	}


	$self->SUPER::init(@_);

	$self->{www_prefix} = $www_prefix;

	if ($self->Exists() && $self->getField('parameters_id')) {
		$self->{manager_login} = $self->getParam('manager_login');
		$self->{manager_passwd} = $self->getParam('manager_passwd');
	}

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;
	my $fields = $$self{dbFields};

	return 'dom_id=' . dbQuote($$fields{dom_id}) . ' AND type="tomcat"';
}


sub dbExistsCondition()
{
	my ($self) = @_;
	return $self->dbGetIdCondition();
}


sub doCreate
{
	my ($self) = @_;

	$self->resetError();

	my %env_vars = ('PSA_PASSWORD' => $$self{'manager_passwd'});

	my @exec_params = (
				'--install-service',
				'--vhost-name=' . $self->{domainName},
				'--manager-login=' . $$self{'manager_login'},
				'--coyote-connector-port=' . $self->{coyote_connector_port},
				'--warp-connector-port=' . $self->{warp_connector_port}
				);

	$self->utilExecEnv('tomcatmng', \%env_vars, @exec_params);
	if ($self->error()) {
		return undef;
	}

	if ($self->getField('status') == 0) {
		my @exec_params = ('--install-vhost', '--vhost-name=' . $self->{domainName},
							'--coyote-connector-port=' . $self->{coyote_connector_port},
							'--warp-connector-port=' . $self->{warp_connector_port});

		if ($self->{www_prefix}) {
			push(@exec_params, '--alias=www.' . $self->{domainName});
		}

		$self->utilExec('tomcatmng', @exec_params);
		if ($self->error()) {
			$self->setError('Unable to install Tomcat vhost: ' . $self->getErrorMsg());
			return undef;
		}
	}

	return 1;
}

###############################################################
#	Separate method used as setParam havy depends on database
#	call it after Store() method... And after it call Store
#	method too...
###############################################################
sub update()
{
	my ($self) = @_;

	if ($self->dbExists()) {
		$self->setParam('manager_login', $self->{manager_login});
		$self->setParam('manager_passwd', $self->{manager_passwd});
	}

	return 1;
}

return 1;

