###################################################################
package BU::PSA::Objects::DatabaseUser;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::Logger;
use BU::PSA::Const;
use BU::PSA::CommonFunc;
use BU::PSA::DB;

use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init($ $ $; $)
{
	my $self = shift @_;
	my ($dbId, $dbName, $dbType, $login, $password_type, $password) = @_;

	my $fields = $self->{dbFields};

	defined($dbId) and $fields->{db_id} = $dbId;# or croak "Argument 'dbId' is undefined.";
	defined($dbName) and $self->{dbName} = $dbName;# or croak "Argument 'dbName' is undefined.";
	defined($dbType) and $self->{dbType} = $dbType;# or croak "Argument 'dbType' is undefined.";
	defined($login) and $fields->{login} = $login or croak "Argument 'login' is undefined.";

	$self->{dbFieldsNames} = { 'login' => 1, 'db_id' => 1, 'account_id' => 0 };

	$self->{dbTable} = 'db_users';
	$self->{Changeable} = '';
	$self->{enable} = 1;

	$self->SUPER::init(@_);

	my $account_id = $self->getField('account_id');
	$self->{objAccount} = new BU::PSA::Objects::Account($account_id ? $account_id : 0);
	$self->{objAccount}->setField('type', $password_type);
	$self->{objAccount}->setField('password', $password);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return sprintf('login=%s AND db_id=%s', dbQuote($fields->{login}), $fields->{db_id});
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return $self->dbGetIdCondition();
}


sub doCheckExists()
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	if ($$self{dbType} eq 'mysql') {
		my $query = dbDoQuery('SELECT User FROM mysql.user WHERE lower(User)=lower(' . dbQuote($fields->{login}) . ')');

		if (my ($tmp) = $query->fetchrow_array) {
			return 1;
		}
	} elsif (($$self{dbType} eq 'postgresql') && $conf{PG_IS_RUNNING} && $conf{'postgresql_admin_login'}) {
		my $query_str = 'select count(*) from pg_user where lower(usename)=lower(' . dbQuote($$fields{'login'}) . ')';
		my $exec_str = sprintf('PGPASSWORD=%s %s/psql -U%s -tc "%s" template1 2>/dev/null',
					shellArgQuote($conf{'postgresql_admin_passwd'}), $conf{PGSQL_BIN_D},
					shellArgQuote($conf{'postgresql_admin_login'}), $query_str);

		my $res = `$exec_str`;
		chomp($res); chomp($res); chomp($res);
		$res =~ s/\s//g;

		if (($res =~ m/^[0-9]+$/) && ($res+0 > 0)) {
			return 1;
		}
	}

	return '';
}

sub Store
{
	my $self = shift @_;

	$self->resetError();

	$self->{objAccount}->Store();

	if ($self->{objAccount}->error()) {
		$self->setError($self->{objAccount}->getErrorMsg());
		return undef;
	}

	$self->setField('account_id', $self->{objAccount}->getDBId());
	$self->SUPER::Store(@_);
}

sub doCreate
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	$self->resetError();

	unless ($self->{dbName}) {
		$self->setError("Database name is not defined.");
		return '';
	}

	if ($$self{dbType} eq 'mysql') {
		### do not do nothing
		return 1;

		unless ($self->{LockSysChanges}) {
			my $dbHost = $self->{enable} ? 'localhost' : '0.0.0.0';

			dbDoQuery('grant ALL PRIVILEGES on ' . $self->{dbName} . '.* to '
						. dbQuote($fields->{login})
						. '@"' . $dbHost . '" identified by ' . dbQuote($self->{objAccount}->getField('password')));

			return dbDoQuery('flush privileges');
		}
	} elsif (($$self{dbType} eq 'postgresql') && $conf{PG_IS_RUNNING}) {
		unless ($self->{LockSysChanges}) {
			my $buff = sprintf('umask 0022; %s/pg_manage add_db_usr %s %s >/dev/null 2>&1',
					$conf{'UTILS_DIR'}, dbQuote($$self{'dbName'}), dbQuote($$fields{'login'}));

			print_stderr("SYSTEM EXEC:($buff)\n") if ($debug > 1);
			if (system($buff) != 0) {
				$self->setError("Unable to add database user: error occured during pg_manage executing: " . util_exec_errormsg());
				return '';
			}

			#################################################################
			#	Because of manual postgres database user restoration during
			#	phase 2	in psarestore, function shouldn't fail if it tries
			#	to create already existing user.
			#################################################################

			######################################################################
			#	Check if the user already exists.
			#	Code inspired by BU::PSA::Objects::DatabaseUser::doCheckExists()
			######################################################################
			my $query = sprintf(
				"SELECT COUNT(*) FROM pg_user WHERE lower(usename)=lower(%s)",
				dbQuote($$fields{'login'}));

			my $command = sprintf(
				"umask 0022; PGPASSWORD=%s %s/psql -U%s -tc \"%s\" template1 2>/dev/null",
				shellArgQuote($conf{'postgresql_admin_passwd' }), $conf{PGSQL_BIN_D},
				shellArgQuote($conf{'postgresql_admin_login'}), $query);

			#################
			#	Looks ugly.
			#################
			my $res = `$command`;
			chomp($res); chomp($res); chomp($res);
			$res =~ s/\s//g;

			if (($res =~ m/^[0-9]+$/) && ($res+0 > 0)) {

				###############################
				#	Such user already exists.
				###############################

			} else {

				############################################
				#	Should create user as it's not exists.
				#	How it can be?
				############################################
				my $query = sprintf(
					"CREATE USER \\\"%s\\\" WITH PASSWORD %s NOCREATEDB NOCREATEUSER",
					$$fields{'login'}, dbQuote($self->{objAccount}->getField('password')));

				$buff = sprintf(
					"umask 0022; PGPASSWORD=%s %s/psql -U%s -tc \"%s\" template1 2>&1 >/dev/null",
					shellArgQuote($conf{'postgresql_admin_passwd'}), $conf{PGSQL_BIN_D},
					$conf{'postgresql_admin_login'}, $query);

				print_stderr("SYSTEM EXEC:($buff)\n") if ($debug > 1);

				if (system($buff) != 0) {
					$self->setError("Unable to add database user: error occured during psql executing.\n");
					return '';
				}
			}
		}
	}

	return 1;
}


sub setEnable($)
{
	my ($self, $enable) = @_;

	$self->{enable} = $enable;

	return 1;
}


sub enable()
{
	my ($self) = @_;

	dbDoQuery('UPDATE mysql.db SET Host="localhost" WHERE Host="0.0.0.0" and User=' . dbQuote($fields->{login}));

	dbDoQuery('UPDATE mysql.user SET Host="localhost" WHERE Host="0.0.0.0" and User=' . dbQuote($fields->{login}));

	$self->{enable} = 1;
	return dbDoQuery('flush privileges');
}


sub disable()
{
	my ($self) = @_;

	dbDoQuery('UPDATE mysql.db SET Host="0.0.0.0" WHERE Host="localhost" and User=' . dbQuote($fields->{login}));

	dbDoQuery('UPDATE mysql.user SET Host="0.0.0.0" WHERE Host="localhost" and User=' . dbQuote($fields->{login}));

	$self->{enable} = '';
	return dbDoQuery('flush privileges');
}


return 1;

