###################################################################
package BU::PSA::Objects::Domain;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::DB;
use BU::FileFunc;
use BU::PSA::CommonFunc;

use BU::PSA::Objects::ObjectWithParams;

@ISA = qw(BU::PSA::Objects::ObjectWithParams);


sub init
{
	my $self = shift @_;
	my ($clientId, $name) = @_;

#print_stdout ("init Domain\n");

	my $fields = $self->{dbFields};

	$fields->{name} = $name;
	$fields->{cl_id} = $clientId;

	$self->{dbFieldsNames} = {
								'name' => 1, 'cl_id' => 0, 'displayName' => 1,
								'cr_date' => 'DATE', 'dns_zone' => 0,
								'status' => 0, 'htype' => 0,
								'real_size' => 0, 'dns_type' => 0,
								'cert_rep_id' => 0, 'limits_id' => 0, 'params_id' => 0
							};

	$$self{action_log_map} = {
					'name' => COMP_DOMAIN_NAME,
#					'dns_zone' => ,
					'status' => COMP_STATUS,
				};

	$self->{dbTable} = 'domains';
	$self->{dbParamsTable} = 'dom_param';
	$self->{dbRefTableIdName} = 'dom_id';
	$self->{dbParamFieldName} = 'param';
	$self->{dbValueFieldName} = 'val';

	$self->{Changeable} = 1;

	$self->{www_prefix} = 1;

	$self->SUPER::init(@_);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'lower(name)=lower(' . dbQuote($fields->{name}) . ')';
}


sub dbExistsCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'lower(name)=lower(' . dbQuote($fields->{name}) . ')';
}


sub setParam($ $)
{
	my ($self, $param, $value) = @_;
	my $fields = $self->{dbFields};

	if ($param eq 'www_prefix') {
		$self->{www_prefix} = $value;
	}

	if ($param ne 'mbox_quota') {
		return $self->SUPER::setParam($param, $value);
	}

	my $saveValue = $self->getParam($param);

	if ($saveValue eq $value) {
		return 1;
	}

	unless ($self->SUPER::setParam($param, $value)) {
		return '';
	}

	if ($value >= 0) {
		unless ($self->utilExecAC('mailmng',
									'--set-domain-disk-quota',
									'--domain-name=' . $fields->{name},
									'--disk-quota=' . ($value+0))) {
			$self->SUPER::setParam($param, $saveValue);
			$self->setError('Unable to set mailbox quota: '. $ExecError);
			return '';
		}
	} else {
		unless ($self->utilExecAC('mailmng',
									'--set-domain-disk-quota',
									'--domain-name=' . $fields->{name},
									'--disk-quota=-1')) {
			$self->SUPER::setParam($param, $saveValue);
			$self->setError('Unable to remove mailbox quota: '. $ExecError);
			return '';
		}
	}

	return 1;
}


sub doCheckExists()
{
	my ($self) = @_;

	return existDir($conf{QMAIL_ROOT_D} . '/mailnames/' . $self->{dbFields}->{name});
}


sub doCreate
{
	my ($self) = @_;

	my $name = $self->{dbFields}->{name};

	unless ($self->utilExecAC('mailmng', ('--add-domain', '--domain-name=' . $name))) {
		$self->setError("Unable to create/update qmail service for domain '$name'.");
#		$self->dbDelete();
#		$self->utilExec('dnsmng', ('update', $name));
		return '';
	}

	unless (-e $conf{DUMP_D} . "/$name") {
		unless ($self->utilExecAC('domainmng', ('--add-domain', '--domain-name=' . $name))) {
			$self->setError("Unable to create dumps directory for domain '$name'.");
			return '';
		}
	}

	return 1;
}


sub doDelete
{
	my ($self) = @_;

	if ($self->utilExec('mailmng', ('--remove-domain', '--domain-name=' . $name))) {
		$self->setError("Unable to delete mail service for domain $name.");
		$self->utilExec('dnsmng', ('update', $name));
		return '';
	}
	
}


#sub dbStore
#{
#	my $self = shift @_;
#
#	if (not (my $res = $self->SUPER::dbStore(@_))) {
#		return $res;
#	}
#
#	unless ($self->dbExists()) {
#		return '';
#	}
#
#	my $query = dbDoQuery('SELECT id FROM dns_recs WHERE dom_id=' . $self->getDBId());
#	
#	unless ($query) {
#		warn_log "Unable to check DNS zone for domain %s\n", $self->getField('name')."\n";
#	} else {
#		unless (my ($id) = $query->fetchrow_array) {
#			unless (createDefaultDNSZone($self->getDBId(), $self->getField('name'), $self->{www_prefix})) {
#				$self->setError('Unable to create default DNS zone.');
#				return 1;
#			}
#		}
#	}
#	
#	return 1;
#}

return 1;

