###################################################################
package BU::PSA::Objects::ObjectWithParams;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::DB;

use BU::Logger;
use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init
{
	my $self = shift @_;

	my @addRequireFields = ('dbParamsTable', 'dbRefTableIdName',
				 'dbParamFieldName', 'dbValueFieldName',
				'dbParamsIdName');

	if (exists($self->{RequiredFields})) {
		my $RequiredFields = $self->{RequiredFields};
		splice(@$RequiredFields, $#RequiredFields + 1, 0, @addRequireFields);
	} else {
		$self->{RequiredFields} = \@addRequireFields;
	}

	$self->{dbParamsIdName} = 'id' unless ($self->{dbParamsIdName});

	$self->SUPER::init(@_);

	return $self;
}

sub setParam($ $)
{
	my ($self, $field, $value) = @_;

	unless ($self->dbExists()) {
		$self->setError("Parameter owner object does not exist.");
		return 0;
	}

	my $params = $self->{params};
	my $fields = $self->{dbFields};

	$params->{$field} = $value;

	my $dbParamsId;
	if (exists($self->{dbParamsIdName}) && ('id' ne $self->{dbParamsIdName})) {
		$dbParamsId = $fields->{$self->{dbParamsIdName}};
	} else {
		$dbParamsId = $self->getDBId();
	}

	if ($dbParamsId) {
		dbDoQuery('REPLACE INTO ' . $self->{dbParamsTable} . '(' . $self->{dbRefTableIdName}
				. ', ' . $self->{dbParamFieldName} . ', '. $self->{dbValueFieldName}
				. ') VALUES('
				. $dbParamsId . ', '. dbQuote($field) . ', ' . dbQuote($value) . ')');
	} else {
    		my $buff = sprintf('INSERT INTO %s(%s, %s) VALUES(%s, %s)', $self->{dbParamsTable},
				    $self->{dbParamFieldName}, $self->{dbValueFieldName},
				    dbQuote($field), dbQuote($value));
		dbDoQuery($buff);

		####################################
		#	Update parameters_id in object
		####################################
		$self->setField($self->{dbParamsIdName}, dbGetInsertId());
	}

	return 1;
}


sub getParam($)
{
	my ($self, $field) = @_;

	unless ($self->dbExists()) {
		$self->setError("Object, which owns the parameter, doesn't exist.");
		return undef;
	}

	my $fields = $self->{dbFields};

	unless ($fields->{$self->{dbParamsIdName}}) {
		return undef;
	}

	my $params = $self->{params};

	if (exists $params->{$param}) {
		return $params->{$param};
	}

	my $query = dbDoQuery('SELECT '. $self->{dbValueFieldName} .' FROM '. $self->{dbParamsTable}
						. ' WHERE ' . $self->{dbRefTableIdName} . '=' . $fields->{$self->{dbParamsIdName}}
						. ' AND ' . $self->{dbParamFieldName} . '=' . dbQuote($field));

	if (($value) = $query->fetchrow_array) {
		$params->{$param} = $value;
		return $value;
	}

	return undef;
}


return 1;

