###################################################################
package BU::PSA::Objects::ProtectedDir;
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;

use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init
{
	my $self = shift @_;
	my ($domainId, $domainName, $ownerLogin, $name, $ssl, $non_ssl) = @_;

	my $fields = $self->{dbFields};

	defined($domainId) and $$fields{'dom_id'} = $domainId or croak "Argument 'domainId' is not defined.";
	defined($domainName) and $self->{domainName} = $domainName or croak "Argument 'domainName' is not defined.";
	defined($ownerLogin) and $self->{ownerLogin} = $ownerLogin or croak "Argument 'domainName' is not defined.";
	$$fields{'path'} = $name; # path can be empty it means '/'
	defined($ssl) and $$fields{'ssl'} = $ssl or croak "Argument 'ssl' is not defined.";
	defined($non_ssl) and $$fields{'non_ssl'} = $non_ssl or croak "Argument 'non_ssl' is not defined.";

	$self->{dbFieldsNames} = {'ssl' => '', 'non_ssl' => '', 'realm' => '', 'path' => 1, 'dom_id'=>1};

	$self->{dbTable} = 'protected_dirs';
	$self->{Changeable} = 1;

	$self->SUPER::init(@_);

	return $self;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	return 'path=' . dbQuote($fields->{path})
		.  ' and dom_id=' . $fields->{dom_id};
}


sub dbExistsCondition()
{
	my ($self) = @_;

	return $self->dbGetIdCondition();
}


sub doCheckExists
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	my $non_ssl_path = $conf{HTTPD_VHOSTS_D} . '/' . $self->{domainName} . "/httpdocs/" . $fields->{path};
	my $ssl_path = $conf{HTTPD_VHOSTS_D} . '/' . $self->{domainName} . "/httpsdocs/" . $fields->{path};

	return (-d $non_ssl_path) || (-d $ssl_path);
}


sub doCreate
{
	my ($self) = @_;

	my $fields = $self->{dbFields};

	$self->resetError();

	my @types = ('ssl', 'non_ssl');

	foreach $type (@types) {
		if ($$fields{'type'} ne 'true') {
			next;
		}

		my $ht_dir = ($type eq 'ssl') ? 'httpdocs' : 'httpsdocs';
		my $path = $conf{HTTPD_VHOSTS_D} . '/' . $self->{domainName} . "/$ht_dir/" . $fields->{path};

		unless (-d $path) {
			unless ($self->utilExec('pdmng', '--add-protection', '--type=' . $ht_dir,
									'--vhost-name=' . $self->{domainName},
									'--user-name=' . $self->{ownerLogin},
									'--directory=/' . $fields->{path}))	{
				$self->setError('Unable to add protection to directory (' . $path . ').');
				return '';
			}
		}
	}

	return 1;
}


return 1;

