###################################################################
package BU::PSA::Objects::SysUser; # psa_3.0
###################################################################

require 5.004;
require Exporter;
use Carp;
use BU::PSA::Const;
use BU::PSA::DB;
use BU::FileFunc;
use BU::PSA::CommonFunc;

use BU::PSA::Objects::Object;

@ISA = qw(BU::PSA::Objects::Object);


sub init($ $ $; $)
{
	my $self = shift @_;
	my ($login, $passwd, $passwd_type, $home, $shell, $quota) = @_;

	my $fields = $self->{dbFields};

	defined($login) and $fields->{login} = $login or croak "Argument 'login' is undefined.";
#	defined($account_id) and $fields->{account_id} = $account_id or croak "Argument 'account_id' is undefined.";
	defined($home) and $fields->{home} = $home or croak "Argument 'home' is undefined.";
	defined($shell) and $fields->{shell} = $shell ? $shell : ''; # or croak "Argument 'shell' is undefined.";
	defined($quota) and $fields->{quota} = $quota;

	$self->{dbFieldsNames} = { 'login' => 1, 'account_id' => 0, 'home' => 1, 'shell' => 0, 'quota' => 0 };

	$self->{dbTable} = 'sys_users';
	$self->{Changeable} = undef;

	$$self{action_log_map} = {
		'login' => COMP_SYSUSER_NAME,
		'shell' => COMP_SYSTEM_SHELL,
		'quota' => COMP_HARD_DISK_QUOTA 
	};

	$self->SUPER::init(@_);

	my $account_id = $self->getField('account_id');
	$self->{objAccount} = new BU::PSA::Objects::Account($account_id ? $account_id : 0);
	$self->{objAccount}->setField('type', $passwd_type);
	$self->{objAccount}->setField('password', $passwd);

	return $self;
}


sub getChangedComponents
{
	my $self = shift @_;

	my $diff1 = $self->SUPER::getChangedComponents(@_);
	my $diff2 = $self->{objAccount}->getChangedComponents();

	my %diff = (%$diff1, %$diff2);

#print "DEBUG:SysUser\n";
#while (my ($component, $tmp) = each(%diff)) {
#	print "($component)\n";
#}

	return \%diff;
}


sub dbGetIdCondition()
{
	my ($self) = @_;

	return dbExistsCondition();
}


sub dbExistsCondition()
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	return 'lower(login)=lower(' . dbQuote($fields->{'login'}) . ')';
}


sub doCheckExists()
{
	my ($self) = @_;
	my $fields = $self->{dbFields};

	if ($self->utilExec('usermng', ('exist', $fields->{'login'}))) {
		return 1;
	}

	return '';
}


sub Store
{
	my $self = shift @_;

	$self->resetError();

	$self->{objAccount}->Store();

	if ($self->{objAccount}->error()) {
		$self->setError($self->{objAccount}->getErrorMsg());
		return undef;
	}

	$self->setField('account_id', $self->{objAccount}->getDBId());
	$self->SUPER::Store(@_);
}


sub doCreate()
{
	my ($self) = @_;

	$self->resetError();

	my %env = ();
	my $password = $self->{objAccount}->getField('password');
	my $password_type = $self->{objAccount}->getField('type');

	if ($password_type eq 'plain') {
		$env{PSA_PASSWORD} = $password;
	} else {
		$env{PSA_CRYPTED_PASSWORD} = $password;
	}

	my $ret = $self->utilExecEnv('usermng', \%env, ('add', $self->getField('login'), $self->getField('home')));
	unless ($ret) {
		$self->setError('Unable to create system user: an error occured during execution usermng'
						. (util_exec_error() ? ': ' . util_exec_errormsg() : '.'));
		return '';
	}

#	$crypt_passwd = crypt($self->getField('passwd'), genSaltForCrypt());
#	setPasswd($self->getField('login'), $crypt_passwd);

	if ($conf{HARD_DISK_QUOTA_IS_SUPPORTED}) {
		my $quota_in_MB = int($self->getField('quota') / (1024 * 1024));

		my $ret = $self->utilExec('usermng', ('quota', $self->getField('login'), $quota_in_MB));
		unless ($ret) {
			$self->setError('Unable to set quota for system user: an error occured during execution usermng'
							. (util_exec_error() ? ': ' . util_exec_errormsg() : '.'));
			return '';
		}
	} else {
		$self->setField('quota', 0);
	}

	return 1;
}


sub doDelete()
{
	my ($self) = @_;

	$self->resetError();

	return $self->utilExecAC('usermng', ('del', $self->getField('login')));
}

