package BU::PSA::Restore::Admin;

require 5.004;
require Exporter;

use Digest::MD5 qw(md5_hex);
use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::Restore::ActionLog;
use BU::PSA::Map;
use BU::PSA::DB;
use BU::XML::CommonFunc;
use BU::PSA::Restore::CommonData qw(restoreReports restoreCustomButtons);

@ISA = qw(Exporter);
@EXPORT = qw(restoreAdmin);

sub restoreAdmin($; $)
{
	my ($adminNode, $map_fields) = @_;

	if ($PRETTYPRINT) {
		print_cl('Admin preferences');
	} else {
		printf_stdout("%s admin preferences.\n", $ProcessName);
	}

	# get Admint login and password
	my ($loginNode) = $adminNode->xql('./login');
	unless ($loginNode) {
		print_stderr("Unable to get login info for PSA admin.\n");
		return '';
	}

	my $login = u2a($loginNode->getAttribute('name'));
	my $passwd = u2a($loginNode->getAttribute('password'));

	if ($flags{RESTORE_MODE} eq 'CHECK') {

		foreach my $logoNode ($adminNode->xql('./params/logo[@file != ""]')) {
			my $fake_file_name = md5_hex(rand());
			$logoNode->setAttribute('destfile', $conf{LOGOS_REPOSITORY_D} . "/$fake_file_name");

			$logoNode->setAttribute('fake', $fake_file_name);
        	hashXMLNodeByCID($logoNode);
		}

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		unless (defined($map_fields)) {
			$map_fields = \%adm_card_fields_rev;
		}

		# restore admin card information
		foreach my $item ($adminNode->xql('./card/*')) {
			if (exists($map_fields->{$item->getNodeName})) {
				my $fieldName = $map_fields->{$item->getNodeName()};
				next if ($fieldName eq '');
				my $textNode = $item->getFirstChild;
				putMiscParam($fieldName, $textNode ? u2a($textNode->toString) : '');
			} else {
				printf_stderr("Unable to retrieve admin card field '%s'\n", $fieldName);
			}
		}

		#restore reports for admin
		restoreReports($adminNode, 0, $const{IS_ADMIN});

		# restore admin logo and logolink
		if (my ($paramsNode) = $adminNode->xql('./params')) {

			my ($paramsNodesList) = $paramsNode->getAttributes();

			for (my $i=0; $i<$paramsNodesList->getLength; $i++) {
				my $paramNode = $paramsNodesList->item($i);
				putMiscParam($paramNode->getNodeName(), u2a($paramNode->getNodeValue()));
			}

			if (my ($logoNode) = $paramsNode->xql('./logo')) {
				my $link = $logoNode->getAttribute('url');
				my $showMode = $logoNode->getAttribute('showMode');

				my $file = undef;
				my $fake = undef;
				if (-e $logoNode->getAttribute('destfile')) {
					$file = u2a($logoNode->getAttribute('file'));
					$fake = $logoNode->getAttribute('fake');
				}

				if ($file || $link) {
					unless ($file) {
						printf_stderr("Logo screen will be turned off in server preferences because the file %s was not restored\n", $file);
					}

					my $logo_id = getMiscParam('logo_id');
					unless ($logo_id) {
						dbDoQuery("INSERT INTO Logos(name, fake, url, showMode) VALUES('$file', '$fake', '$link', '$showMode')");
						$logo_id = dbGetInsertId();
						putMiscParam('logo_id', $logo_id);
					} else {
						dbDoQuery("UPDATE Logos set name='$file', fake='$fake', url='$link', showMode='$showMode' WHERE id=$logo_id");
					}
				}
			}
		}
	}

	restoreCustomButtons($adminNode, 0, $const{'IS_ADMIN'}, 1);

	return 1;
}

return 1;

