package BU::PSA::Restore::AnonFTP;

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::AnonFTP;

#use XML::DOM;

@ISA = qw(Exporter);
@EXPORT = qw(restoreAnonFTP);

sub restoreAnonFTP($ $ $ $)
{
	my ($domainId, $domainName, $ftpNode, $ip_address_id) = @_;

	my $status = $ftpNode->getAttribute('status');

	printf_stdout("%s anonymous ftp service.\n", $ProcessName);

	my $ftp = new BU::PSA::Objects::AnonFTP($domainId, $domainName);

	unless ($ftp) {
		printf_stderr("Unable to create \"BU::PSA::Database\" object\n");
		return '';
	}

	if ($ftp->error()) {
		printf_stderr("Error in AnonFTP object: %s\n", $ftp->getErrorMsg());
		return '';
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {

		$ftp->setField('status', $status) if $status;

		my $query = dbDoQuery("SELECT count(*) FROM hosting h, anon_ftp a WHERE h.dom_id!='$domainId' AND a.status='true' AND a.dom_id=h.dom_id AND h.ip_address_id='$ip_address_id'");
		if ($query) {
			my ($number_of_domains) = $query->fetchrow_array();
			if ($number_of_domains + 0 > 0) {
				print_stderr("Domain '$domainName' already has turned on anonymous FTP on the same ip address\n");
				print_stderr("Turn off AnonFTP for domain '$domainName'\n");
				$ftp->setField('status', 'false');
			}
		} else {
			print_stderr("Unable to check AnonFTP existing on exclusive ip for domain '$domainName': sql error\n");
			print_stderr("Turn off AnonFTP for domain '$domainName'\n");
			$ftp->setField('status', 'false');
		}

		my @anon_ftp_params=('incoming', 'max_conn', 'bandwidth',
							'incoming', 'incoming_readable', 'incoming_subdirs',
							'display_login', 'login_text');
		foreach my $param (@anon_ftp_params) {
			$ftp->setField($param, u2a($ftpNode->getAttribute($param)));
		}

		if (my ($sizeNode) = $ftpNode->xql('./size')) {
			my $quota = $sizeNode->getAttribute('limit');
			if ($quota ne '') {
				$ftp->setField('quota', $quota);
			} else {
				print_stderr("ERROR: unable to retrive anonymous ftp incoming dir quota from dump file\n");
			}
		}

		$ftp->Store();

		if ($ftp->error()) {
			printf_stderr("ERROR: Failed to enable anonymous FTP service on this domain: %s\n", $ftp->getErrorMsg());
			return '';
		}
		chmod 0777, $conf{HTTPD_VHOSTS_D} . "/$domainName/anon_ftp/incoming";
	}

	return 1;
}


return 1;

