package BU::PSA::Restore::DNSRec;

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Restore::IPMapping;
use BU::PSA::Objects::DNSRec;

#use XML::DOM;

@ISA = qw(Exporter);
@EXPORT = qw(restoreDNSRec);

sub restoreDNSRec($ $; $)
{
	my ($domainId, $dnsRecNode, $new_ip_address_id) = @_;

	my $host;
	my $val;
	my $option;
	my $type;

	my $type = $dnsRecNode->getAttribute('type');

	if ($type eq 'A') {
		$host = u2a($dnsRecNode->getAttribute('name'));
		if ($new_ip_address_id && (my $ip_address = getIPAddressById($new_ip_address_id))) {
			$val = $ip_address;
		} else { 
			my $ip_address_id = $dnsRecNode->getAttribute('ip_address_id');
			$val = ($ip_address_id) ? getIPAddressById($ip_address_id) : $dnsRecNode->getAttribute('ip');
		}
	} elsif ($type eq 'NS') {
		$host = u2a($dnsRecNode->getAttribute('name'));
		$val = u2a($dnsRecNode->getAttribute('ns'));
	} elsif ($type eq 'PTR') {
		if ($new_ip_address_id && (my $ip_address = getIPAddressById($new_ip_address_id))) {
			$host = $ip_address;
		} else { 
			my $ip_address_id = $dnsRecNode->getAttribute('ip_address_id');
			$host = ($ip_address_id) ? getIPAddressById($ip_address_id) : $dnsRecNode->getAttribute('ip');
		}
		$val = u2a($dnsRecNode->getAttribute('name'));
		$option = $dnsRecNode->getAttribute('netmask');
	} elsif ($type eq 'CNAME') {
		$host = u2a($dnsRecNode->getAttribute('name'));
		$val = u2a($dnsRecNode->getAttribute('cname'));
	} elsif ($type eq 'MX') {
		$host = u2a($dnsRecNode->getAttribute('name'));
		$val = u2a($dnsRecNode->getAttribute('mx'));
		$option = $dnsRecNode->getAttribute('priority');
	} elsif( $type eq 'TXT' ) {
		$host = u2a($dnsRecNode->getAttribute('name'));
		$val = u2a($dnsRecNode->getAttribute('text'));
	} else {
		print_stderr("Unable to add a DNS record: unknown record type.\n");
		return '';
	}

	my $dnsRec = new BU::PSA::Objects::DNSRec(
								$domainId,
								$host,
								$type,
								$val,
								$option,
								$dnsRecNode->getAttribute('time_stamp')
							);

	unless ($dnsRec) {
		printf_stderr("Unable to create DNS record of '%s' type\n", $type);
		return '';
	}

	if ($dnsRec->error()) {
		printf_stderr("Unable to create DNS record: %s\n", $dnsRec->getErrorMsg());
		return '';
	}

	if ($new_ip_address_id && (my $ip_address = getIPAddressById($new_ip_address_id))) {
		if (($type eq 'A')) {
			$dnsRec->setField('val', $ip_address);
		} elsif ($type eq 'PTR') {
			$dnsRec->setField('host', $ip_address);
		}
	}

	$dnsRec->Store();

	if ($dnsRec->error()) {
		printf_stderr("Unable to save DNS record of %s type: %s\n",	$type, $dnsRec->getErrorMsg());
		return '';
	}

	return 1;
}


return 1;

