#####################################################
package BU::PSA::Restore::DSMailLists;
#####################################################

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::MailList;
use BU::XML::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(restoreDSMailLists);

sub restoreDSMailLists
{
	my ($base_node, $domainId, $domainName) = @_;

	unless ($conf{'MAILMAN_IS_INSTALLED'}) {
		# restore MailLists
		foreach my $node ($base_node->xql('./mail_list')) {
			$node->setAttribute('restore', 'false');
		}
		return 1;
	}

	my $status = $base_node->getAttribute('status');
	$status = $status ^ ($status & $const{'STATUS_WITH_PARENT'});

	my $isExist = 0;
	if ($domainId) {
		my $query = dbDoQuery('SELECT count(*) FROM DomainServices WHERE type=\'maillists\' AND dom_id=' . dbQuote($domainId));
		unless (($isExist) = $query->fetchrow_array()) {
			printf_stderr("Unable to define 'mail' domain service existing from database.\n");
			return '';
		}
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		unless ($isExist) {
			dbDoQuery('INSERT INTO DomainServices(dom_id, type, status) VALUES('
					. dbQuote($domainId) . ", 'maillists', $status)");
		} else {
			dbDoQuery("UPDATE DomainServices SET status=$status WHERE type=\"maillists\" AND dom_id=$domainId");
		}
	}

	# restore MailLists
	foreach my $node ($base_node->xql('./mail_list')) {
		restoreMailList($node, $domainId, $domainName, $status);
	}

	return 1;
}


sub restoreMailList
{
	my ($base_node, $domainId, $domainName, $ds_status) = @_;

	my $mlName = $base_node->getAttribute('name');
	my $mlStatus = $base_node->getAttribute('status');

	$mlStatus = $mlStatus ^ ($mlStatus & $const{'STATUS_WITH_PARENT'});

	if ($PRETTYPRINT) {
		print_sl("Mail list '$mlName' for domain '$domainName'");
	} else {
		printf_stdout("%s mail list '%s' for domain '%s'\n", $ProcessName, $mlName, $domainName);
	}

	my $isExist = undef;

	$mlObj = new BU::PSA::Objects::MailList($domainId, $mlName, $domainName);
	if ($mlObj->error()) {
		printf_stderr("Error MailList object creating: %s.\n", $mlObj->getErrorMsg());
		return '';
	}

	my $config_dir = $conf{MAILMAN_ROOT_D} . "/lists/$mlName";
	my $archive_private_dir = $conf{MAILMAN_ROOT_D} . "/archives/private/$mlName";

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		my $query = undef;

		### check for mailname with such name
		$query = dbDoQuery('SELECT id FROM mail WHERE mail_name=' . dbQuote($mlName));
		if (my ($tmp_id) = $query->fetchrow_array()) {
			printf_stderr("Unable to restore maillist '%s' for domain '%s': mail name with such name already exists.\n",
							$mlName, $domainName);
			markCollision($base_node);
			return '';
		}

		### check for mailalias with such name
		$query = dbDoQuery('SELECT id FROM mail_aliases WHERE alias=' . dbQuote($mlName));
		if (my ($tmp_id) = $query->fetchrow_array()) {
			printf_stderr("Unable to restore maillist '%s' for domain '%s': mail alias with such name already exists.\n",
							$mlName, $domainName);
			markCollision($base_node);
			return '';
		}

		### check for maillist with such name on other domain
		$query = dbDoQuery('SELECT d.name, ml.id FROM MailLists ml, domains d WHERE d.id=ml.dom_id AND ml.name=' . dbQuote($mlName));
		if (($mlDomainName, $ml_id) = $query->fetchrow_array()) {
			$isExist = 1;
			if ($mlDomainName ne $domainName) {
				printf_stderr("Unable to restore maillist '%s' for domain '%s': mail list with such name already exists on domain '%s'.\n",
								$mlName, $domainName, $mlDomainName);

				markCollision($base_node);
			}
		}

		if ($isExist) {
			$base_node->setAttribute('exists', 'true');
		}

		my $suffix = randFileName('tmp');
		if (my ($node) = $base_node->xql('./config_dir')) {
			$node->setAttribute('destdir', "$config_dir.$suffix");
        	hashXMLNodeByCID($node);
		}
		if (my ($node) = $base_node->xql('./archive_private_dir')) {
			$node->setAttribute('destdir', $archive_private_dir.$suffix);
        	hashXMLNodeByCID($node);
		}
		if (my ($node) = $base_node->xql('./archive_private_mbox_dir')) {
			$node->setAttribute('destdir', "$archive_private_dir.mbox.$suffix");
        	hashXMLNodeByCID($node);
		}

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if ($base_node->getAttribute('collision') eq 'true') {
			printf_stderr("Mail list '%s' restoration is skipped.\n", $mlName);
			return 1;
		}

		my %dirs_list = (
			'./config_dir' => $config_dir,
			'./archive_private_dir' => $archive_private_dir,
			'./archive_private_mbox_dir' => "$archive_private_dir.mbox"
		);

		$mlObj->setField('status', $mlStatus | ($ds_status ? $const{'STATUS_WITH_PARENT'} : 0));
		$mlObj->Store();
		if ($mlObj->error()) {
			printf_stderr("Error during mail name creating: %s.\n", $mlObj->getErrorMsg());
			foreach my $node_path (keys %dirs_list) {
				if (my ($node) = $base_node->xql($node_path)) {
					$dest_tmpdir = $node->getAttribute('destdir');
					if (-d $dest_tmpdir) {
						system("rm -rf $dest_tmpdir");
					}
				}
			}
			return '';
		}

		unless (-d $conf{'OLD_MAILMAN_ROOT_D'}) {
			system(sprintf('ln -s %s %s', $conf{'MAILMAN_ROOT_D'}, $conf{'OLD_MAILMAN_ROOT_D'}));
		}

		foreach my $node_path (keys %dirs_list) {
			my $dest_dir = $dirs_list{$node_path};
			my $dest_tmpdir = undef;
			if (my ($node) = $base_node->xql($node_path)) {
				$dest_tmpdir = $node->getAttribute('destdir');
				unless (-d $dest_tmpdir) {
					next;
				}
				if (-d $dest_dir) {
					system("rm -rf $dest_dir");
				}
				system("mv $dest_tmpdir $dest_dir");

#				if (-d $dest_dir) {
#					### change VHOSTS dir 
#					if (open(fd_list, "find $dest_dir -type f")) {
#						my $old_vhosts = $conf{'OLD_MAILMAN_ROOT_D'};
#						my $new_vhosts = $conf{'MAILMAN_ROOT_D'};
#						$old_vhosts =~ s/\//\\\//g;
#						$new_vhosts =~ s/\//\\\//g;
#
#						my $suffix = randFileName('tmp');
#						my $buff = "grep '%s' %s >/dev/null 2>&1 && sed -e 's/%s/%s/g' < %s > %s && mv %s %s";
#
#						while(<fd_list>) {
#							chomp($_);
#							my $fname = $_;
#							my $tmp_fname = "$fname.$suffix";
#							system(sprintf($buff, $old_vhosts, $fname, $old_vhosts, $new_vhosts, $fname, $tmp_fname, $tmp_fname, $fname));
#						}
#					}
#				}
			}
		}
	}
}

return 1;


