#####################################################
package BU::PSA::Restore::DomainLevelUser;
#####################################################

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::PSA::CommonFunc;
use BU::PSA::Restore::ActionLog;
use BU::PSA::DB;
use BU::XML::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(restoreDomainLevelUser);


sub restoreDomainLevelUser
{
	my ($domainId, $domainName, $baseNode) = @_;

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		# empty
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		my $status = $baseNode->getAttribute('status');

		if ($status eq 'true') {
			my $query = dbDoQuery('SELECT id FROM clients WHERE login=' . dbQuote($domainName));
			unless ($query) {
				print_stderr("Unable to retrieve data from client's table in the database.\n");
			} elsif ($query->fetchrow_array()) {
				$status = 'false';
				printf_stderr("Domain level user account is turned off for the domain %s, ".
					 "because on the target system PSA client with such name already exists\n",
					 $domainName);
			}
		}	

		my $card_id = undef;
		if (my ($cardNode) = $baseNode->xql('./card')) {
			my %new_card_fields = ( 
				'pname' => 'personalName', 'cname' => 'companyName', 'phone' => 'phone', 'fax' => 'fax',
				'email' => 'email', 'address' => 'address', 'city' => 'city', 'state' => 'state',
				'zip' => 'zip', 'country' => 'country'
			);

			my $nodesList = $cardNode->getChildNodes();

			my $query = dbDoQuery('SELECT dlu.card_id FROM dom_level_usrs dlu, domains d WHERE d.id=dlu.dom_id AND d.name=' . dbQuote($domainName));
			if (($card_id) = $query->fetchrow_array()) {
				my $buff = XMLNodesList2SQL('REPLACE INTO Cards(id, %s) VALUES('.$card_id.', %s)', $nodesList,
											{decode_base64 => 1, map_fields => \%new_card_fields});
				dbDoQuery($buff);
			} else {
				my $buff = XMLNodesList2SQL('INSERT INTO Cards(%s) VALUES(%s)', $nodesList,
											{decode_base64 => 1, map_fields => \%new_card_fields});

				dbDoQuery($buff);
				$card_id = dbGetInsertId();
			}
		}

		my $perm_id = undef;
		if (my ($node) = $baseNode->xql('./permissions')) {
			my $query = dbDoQuery('SELECT perm_id FROM dom_level_usrs WHERE dom_id=\'' . $domainId . '\'');
			($perm_id) = $query->fetchrow_array() if $query;
			my $attrsNodesList = $node->getAttributes();
			for (my $i=0; $i<$attrsNodesList->getLength; $i++) {
				my $sn = $attrsNodesList->item($i);

				if ($perm_id) {
					dbDoQuery(sprintf('REPLACE INTO Permissions(id, permission, value) VALUES("%s", "%s", "%s")',
								$perm_id, $sn->getNodeName(), $sn->getNodeValue()));
				} else {
					dbDoQuery(sprintf('INSERT INTO Permissions(permission, value) VALUES("%s", "%s")',
										$sn->getNodeName(), $sn->getNodeValue()));
					$perm_id = dbGetInsertId();
				}
			}
		}

		if (my ($loginNode) = $baseNode->xql('./login')) {
			my $password = u2a($loginNode->getAttribute('password'));
			my $password_type = u2a($loginNode->getAttribute('PW_TYPE'));
			my $query = dbDoQuery('SELECT account_id FROM dom_level_usrs WHERE dom_id=\'' . $domainId . '\'');
			my ($account_id) = $query->fetchrow_array() if $query;
			if ($account_id) {
				dbDoQuery("UPDATE accounts SET type='$password_type', password=" . dbQuote($password) . " WHERE id='$account_id'");
			} else {
				dbDoQuery("INSERT INTO accounts(type, password) VALUES('$password_type', " . dbQuote($password) . ")");
				$account_id = dbGetInsertId();
			}
			dbDoQuery("REPLACE INTO dom_level_usrs(dom_id, state, card_id, account_id, perm_id)"
						. " values($domainId, '$status'"
						. ', ' . ($card_id ? $card_id : 'NULL')
						. ', ' . ($account_id ? $account_id : 0)
						. ', ' . ($perm_id ? $perm_id : 0) . ')'
						);

			
		}
	} # end of RESTORE if

	return 1;
}

return 1;

