package BU::PSA::Restore::Mail;

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::Account;
use BU::PSA::Objects::MailName;
use BU::XML::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(restoreMailName restoreAutoresponder);

sub restoreMailName($ $ $)
{
	my ($domainId, $domainName, $mailNode) = @_;

	my $loginNode = undef;

	unless (($loginNode) = $mailNode->xql('./login')) {
		print_stderr("Unable to retrieve mail login\n");
		return '';
	}

	##########################################
	#
	#	Get mailnames form (simple/compound)
	#	to check mailnames existence.
	#
	##########################################
	if (not exists $conf{SHORT_POP3_NAMES}) {
		$conf{SHORT_POP3_NAMES} = getMiscParam('allow_short_pop3_names');
		$conf{SHORT_POP3_NAMES} = 
			($conf{SHORT_POP3_NAMES} eq 'disabled') ? 'false' : 'true';
	}

	my $loginName = u2a($loginNode->getAttribute('name'));

	printf_stdout("%s mailname '%s'\n", $ProcessName, $loginName);

	my ($servicesNode) = $mailNode->xql('./services');

	# if $domainId is undefined than domain is not exist and mail name is not exist the same!
	my $isExist = '';
	my $mail = undef;
	if ($domainId) {
		my $password = u2a($loginNode->getAttribute('password'));
		my $password_type = $loginNode->getAttribute('PW_TYPE');

		my $query = dbDoQuery('SELECT mail_name FROM mail WHERE dom_id= ' . dbQuote($domainId) . ' AND lower(mail_name)=lower(' . dbQuote($loginName) . ')');
		if (my ($tmp) = $query->fetchrow_array()) {
			############################################
			### FETCH AND USE MAIL NAME AS IN DATABASE
			$loginName = $tmp;
			$loginNode->setAttribute('name', a2u($loginName));
			############################################
		}

		$mail = new BU::PSA::Objects::MailName($domainId, $domainName, $loginName, $password_type, $password);
		if ($mail->error()) {
			print_stderr($mail->getErrorMsg());
			return '';
		}
		$isExist = $mail->Exists();
		unless ($isExist) {
			$mail->setField('account_id', '0');
		}
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		if ($isExist) {
			$mailNode->setAttribute('exists', 'true');

			unless ($flags{OVERWRITE}) {
				printf_stderr("Restoring for the '%s' mail user is impossible: ".
							 "mail user with such name already exists.\n",
							 $loginName);

				markCollision($mailNode);
				return '';
			}
		} else {
			$conf{'DOMAIN_MAILNAMES_USAGE'}++;
		}

		if (my ($loginNode) = $mailNode->xql('./login')) {
			$mailNode->setAttribute('destdir', $conf{QMAIL_MAILNAMES_D} . "/$domainName/" . u2a($loginNode->getAttribute('name')));
        	hashXMLNodeByCID($mailNode);

			#
			# Assemble path for autoresponder attachments
			#
			foreach my $attachNode ($mailNode->xql('./autoresponder/attach')) {
				$attachNode->setAttribute('destfile', $conf{QMAIL_MAILNAMES_D}
										. "/$domainName/" . u2a($loginNode->getAttribute('name'))
										. '/@attachments/' . u2a($attachNode->getAttribute('file'))
										);
       			hashXMLNodeByCID($attachNode);
			}

			foreach my $attachNode ($mailNode->xql('./repository/file')) {
				$attachNode->setAttribute('destfile', $conf{QMAIL_MAILNAMES_D}
										. "/$domainName/" . u2a($loginNode->getAttribute('name'))
										. '/@attachments/' . u2a($attachNode->getAttribute('name'))
										);
       			hashXMLNodeByCID($attachNode);
			}
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if ($mailNode->getAttribute('collision') eq 'true') {
			printf_stderr("'%s' mail user restoration is skipped.\n", $loginName);
			return '';
		}

		########################################################
		#	Restore control panel access settings for mailuser
		########################################################
		$mail->setField('cp_access', $loginNode->getAttribute('cp_access'));

		if ($servicesNode) {
			my $password = u2a($loginNode->getAttribute('password'));
			my $password_type = $loginNode->getAttribute('PW_TYPE');

			#####################################################
			#
			#	Check that there is no similar mailnames on the 
			#	server if short pop3 names are allowed
			#
			#####################################################
			if ($conf{SHORT_POP3_NAMES} eq 'true') {
				my $tmp = ($password_type ne 'plain') ? '1' : '0';
				my $query = dbDoQuery('SELECT m.id FROM mail m, domains d, accounts a WHERE'
								. ' m.account_id=a.id and d.id=m.dom_id and d.name!=' . dbQuote($domainName)
								. ' and m.mail_name=' . dbQuote($loginName)
								. ' and ((a.password=' . dbQuote($password)
								. ' and a.type="plain")'
								. ' or (a.type="crypt") or (' . $tmp . '=1))'
								);

				if (!$query || $query->fetchrow_array()) {
					$mail->setField('postbox', 'false');
					printf_stderr("Mailbox collision detected for mailname '%s', mailbox turned off.\n", $loginName);

				} else {

					$mail->setField('postbox', $servicesNode->getAttribute('postbox'));
				}

			} else {

				#######################################################
				#	As there was not detected collisions there are no
				#	reasons to prevent mailname from being restored.
				#######################################################
				$mail->setField('postbox', $servicesNode->getAttribute('postbox'));
			}

			if ($mail->getField('postbox') eq 'true') {
				my $account = new BU::PSA::Objects::Account($mail->getField('account_id'));
				unless ($account) {
					printf_stderr("Unable to create %s object for domain %s\n", "BU::PSA::Objects::Account", $domainName);
					return '';
				}

				$account->setField('type', $loginNode->getAttribute('PW_TYPE'));
				$account->setField('password', $postbox_password);

				$account->Store();

				if ($account->error()) {
					print_stderr("An error occured during mail user account creating: " . $account->getErrorMsg());
				} else {
					$mail->setField('account_id', $account->getDBId());
				}
			}

			$mail->setField('autoresponder', $servicesNode->getAttribute('autoresponder'));

			if (my ($forwardNode) = $servicesNode->xql('./forwarding[@redirect]')) {
				$mail->setField('redirect', $forwardNode->getAttribute('redirect'));
				if (my ($addressNode) = $forwardNode->xql('./email')) {
					my $textNode = $addressNode->getFirstChild;
					$mail->setField('redir_addr', $textNode ? $textNode->toString : '');
				}
			} else {
				$mail->setField('redirect', 'false');
			}

			my ($mailGroupNode) = $servicesNode->xql('./forwarding[@mailgroup]');
			if ($mailGroupNode) {
				$mail->setField('mail_group', $mailGroupNode->getAttribute('mailgroup'));
			} else {
				$mail->setField('mail_group', 'false');
			}

			if (my ($paramsNode) = $mailNode->xql('./params')) {
				my $mbox_quota = $paramsNode->getAttribute('mbox_quota');
				$mail->setField('mbox_quota', $mbox_quota) unless ($mbox_quota eq '');
			}

			$mail->Store();

			if ($mail->error()) {
				print_stderr("Unable to create mailname: %s\n", $mail->getErrorMsg());
				return '';
			}

			####################################
			## restore spamassassin preferences
			####################################
			if ($conf{'SPAMASSASSIN_IS_INSTALLED'}) {
				if (my ($saNode) = $mailNode->xql('./spamassassin')) {
					my $mailname = $loginName . '@' . $domainName;

					dbDoQuery('DELETE FROM sa_list WHERE mailname=' . dbQuote($mailname));
					dbDoQuery('DELETE FROM sa_conf WHERE mailname=' . dbQuote($mailname));

					my $buff = sprintf("INSERT INTO sa_conf(mailname,flt_enabled,rw_subject_tag,hits_required) VALUES(%s,%s,%s,%s)",
									dbQuote($mailname),
									dbQuote($saNode->getAttribute('flt_enabled')),
									dbQuote($saNode->getAttribute('rw_subject_tag')),
									dbQuote($saNode->getAttribute('hits_required'))
								);
					dbDoQuery($buff);

					my @options = ();

					my $flt_enabled = $saNode->getAttribute('flt_enabled');
					if (($flt_enabled eq 'serv') || ($flt_enabled eq 'both')) {
						push @options, '--enable-server-configs';
					}
					if (($flt_enabled eq 'user') || ($flt_enabled eq 'both')) {
						push @options, '--enable-user-configs';
					}

					my $rw_subject_tag = $saNode->getAttribute('rw_subject_tag');
					if ($saNode->getAttribute('rw_subject_tag')) {
						push @options, '--modify-header=' . shellArgQuote($rw_subject_tag);
					}

					my $hits_required = $saNode->getAttribute('hits_required');
					if ($hits_required) {
						push @options, '--sensitivity=' . shellArgQuote($hits_required);
					}

					my $white_pattern = '';
					my $black_pattern = '';
					foreach my $sn ($saNode->xql('./*')) {
						$buff = sprintf("INSERT INTO sa_list(mailname, pattern, color) VALUES(%s, %s, %s)",
									dbQuote($mailname),
									dbQuote($sn->getAttribute('pattern')),
									dbQuote($sn->getAttribute('color'))
								);
						dbDoQuery($buff);

						if ($sn->getAttribute('color') eq 'white') {
							$white_pattern = ($white_pattern ? ',' : '') . $sn->getAttribute('pattern');
						} else {
							$black_pattern = ($black_pattern ? ',' : '') . $sn->getAttribute('pattern');
						}
					}

					if ($white_pattern) {
						push @options, '--add-to-whitelist=' . shellArgQuote($white_pattern);
					}
					if ($black_pattern) {
						push @options, '--add-to-blacklist=' . shellArgQuote($black_pattern);
					}
					push @options, '--mailname=' . shellArgQuote($mail->getField('mail_name') . '@' . $domainName);
#					push @options, 'restart';

					util_exec('spammng', '-e', '--mailname=' . shellArgQuote($mail->getField('mail_name') . '@' . $domainName));
					util_exec('spammng', @options);
				}
			}

			my $mn_id = $mail->getDBId();
			foreach my $mailAliasNode ($mailNode->xql('./aliases/*')) {
				my $aliasName = u2a($mailAliasNode->getAttribute('name'));
				if (util_exec('mailmng', '--add-alias', '--domain-name=' . $domainName,
								'--mailname=' . $loginName, '--alias=' . $aliasName)) {
					dbDoQuery("REPLACE INTO mail_aliases(mn_id, alias) VALUES($mn_id, " . dbQuote($aliasName) . ')');
				} else {
					printf_stderr("Unable to create mail alias '%s' for mailname '%s' on domain '%s': %s\n", $aliasName, $loginName, $domainName, util_exec_errormsg());
				}
			}

			# Mailname created - update mail user parameters, placed in mail node -> params -> parameter
			# at least locale and skin_id are affected by this...
			my $drweb_key_where = undef;
			if (my ($paramsNode) = $mailNode->xql('./params')) {

				foreach my $mailnameParamNode ($paramsNode->xql('./parameter')) {
					my $param = $mailnameParamNode->getAttribute('name');
					my $value = $mailnameParamNode->getAttribute('value');

					if ($param eq 'drweb_direction') {
						$drweb_key_where = $value;
					}

					dbDoQuery(
						"REPLACE INTO mn_param(mn_id, param, val) ".
						"VALUES($mn_id, ".
								dbQuote($param).", ".
								dbQuote($value).")"
					);
				}
			}

			### drweb check on/off
			if ($conf{DRWEB_IS_INSTALLED}) {
				my $result = undef;
				if ($drweb_key_where && ($drweb_key_where ne 'off')) {
					$result = util_exec('drwebmng', '--enable-check',
										'--domain-name=' . $domainName,
										'--mailname=' . $mail->getField('mail_name'),
										'--type=' . $drweb_key_where
										);
				} else {
					$result = util_exec('drwebmng', '--disable-check',
										'--domain-name=' . $domainName,
										'--mailname=' . $mail->getField('mail_name'),
										);
				}
				unless ($result) {
					print_stderr("An error occured during drwebmng executing: " . util_exec_errormsg());
				}
			}
			### end of drweb

			if ($mailGroupNode) {
				foreach my $emailNode ($mailGroupNode->xql('./email')) {
					my $textNode = $emailNode->getFirstChild;
					$mail->addForward($textNode->toString, ($mail->getField('mail_group') eq 'true')) if $textNode;
				}
			}

			# autoresponders information and properties are stored in mail node
			foreach my $respNode ($mailNode->xql('./autoresponder')) {
				restoreAutoresponder($mail, $respNode);
			}
		} else {
			$mail->Store();
		}


		if ($mail->error()) {
			warn $mail->getErrorMsg();
			return '';
		}
	} # end RESTORE if

	return 1;
}


sub restoreAutoresponder($ $)
{
	my ($mail, $respNode) = @_;

	my $ansFreq = '';
	my $memLimit = '';
	my $reply_to = '';
	if (my ($paramsNode) = $respNode->xql('./params')) {
		$ansFreq = $paramsNode->getAttribute('ans_freq');
		$memLimit = $paramsNode->getAttribute('mem_limit');
		$reply_to = $paramsNode->getAttribute('reply_to');
	}

	my $keyWhere = '';
	my $keyString = '';
	if (my ($keyStringNode) = $respNode->xql('./keystr')) {
		$keyWhere = $keyStringNode->getAttribute('where');
		$keyString = $keyStringNode->getFirstChild();
		if ($keyString) {
			$keyString = u2a($keyString->toString);
		}
	}

	my $subject = '';
	my $respText ='';
	if (my ($messageNode) = $respNode->xql('./message')) {
		$subject = u2a($messageNode->getAttribute('subject'));
		$respText = $messageNode->getFirstChild;
		if ($respText) {
			$respText = u2a($respText->toString);
		}
	}


	my $resp = $mail->addAutoResp(
									$respNode->getAttribute('name'),
									$respNode->getAttribute('turn') eq 'true',
									$keyString,
									$keyWhere,
									$subject,
									$respText,
									$ansFreq,
									$memLimit,
									$reply_to
								);

	unless ($resp) {
		printf_stderr("Unable to create autoresponder for mailname '%s': %s\n",
						$mail->getField('mail_name'), $mail->getErrorMsg());
		return '';
	}

	if (my ($forwardNode) = $respNode->xql('./forwarding[@autoresponder = "true"]/email')) {
		my $textNode = $forwardNode->getFirstChild;
		if ($textNode) {
			$resp->setForward($textNode->toString);
		} else {
			print_stderr("An error occured during autoresponder forwarding restoration:"
				." unable to retrive email address.\n");
		}
	} else {
		$resp->setForward('');
	}

	# restore attachments files
	foreach my $attachNode ($respNode->xql('./attach')) {
		my $destfile = $attachNode->getAttribute('destfile');
	
		unless ($destfile) {
			print_stderr("Unable to add attachment file: file name is empty.\n");
		} else {

			unless (-e $destfile)
			{
				printf_stderr("Unable to add attachment file %s: file does not exist.\n", $dest_file);
			} else {
				$resp->addAttachment(u2a($attachNode->getAttribute('file')));
			}
		}
	}

	return 1;
}


return 1;

