package BU::PSA::Restore::PDir;

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::ProtectedDir;
use BU::PSA::Objects::ProtectedDirUser;

#use XML::DOM;

@ISA = qw(Exporter);
@EXPORT = qw(restorePDir restorePDirUser);

sub restorePDir($ $ $ $)
{
	my ($domainId, $domainName, $ownerLogin, $pdirNode) = @_;

	my $ssl = undef;
	my $not_ssl = undef;

	if (my ($servicesNode) = $pdirNode->xql('./services')) {
		$ssl = $servicesNode->getAttribute('ssl');
		$not_ssl = $servicesNode->getAttribute('non_ssl');
	}
	$ssl = 'false' unless $ssl;
	$not_ssl = 'false' unless $not_ssl;

	printf_stdout("%s protected dir '%s'\n", $ProcessName, $pdirNode->getAttribute('path'));

	my $pdir = new BU::PSA::Objects::ProtectedDir (
										$domainId,
										$domainName,
										$ownerLogin,
										$pdirNode->getAttribute('path'),
										$ssl,
										$not_ssl
									);

	unless ($pdir) {
		printf_stderr("Unable to create ProtectedDir object (%s)\n", $domainName);
		return '';
	}

	if ($pdir->error()) {
		warn $pdir->getErrorMsg() . "\n";
		return '';
	}


	if ($flags{RESTORE_MODE} eq 'CHECK') {
#		unless ($pdir->Exists()) {
#			return '';
#		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if (my ($servicesNode) = $pdirNode->xql('./services')) {
			$pdir->setField('ssl', $servicesNode->getAttribute('ssl'));
		}

		if (my ($aliasNode) = $pdirNode->xql('./realm')) {
			my $textNode = $aliasNode->getFirstChild();
			$pdir->setField('realm', $textNode ? u2a($textNode->toString) : '');
		}

		$pdir->Store();

		if ($pdir->error()) {
			warn $pdir->getErrorMsg()."\n";
			return '';
		}

#		my $path_name = 'd..' . $pdirNode->getAttribute('path');
#		$path_name =~ s/\//../g;
#
#		my $full_path = $conf{HTTPD_VHOSTS_D} . "/$domainName/pd/$path_name";
#		system("rm -f $full_path > /dev/null 2>&1");
#
#		print_stdout ("print 'rm -f $full_path'\n") if $debug > 0;
	}

	foreach my $userNode ($pdirNode->xql('./user[@type = "pd_users"]')) {
		restorePDirUser($domainName, $pdir, $userNode);
	}

	return 1;
}


sub restorePDirUser($ $ $)
{
	my ($domainName, $pdir, $userNode) = @_;


	my ($loginNode) = $userNode->xql('./login');
	unless ($loginNode) {
		print_stderr("Unable to retrieve login information for the protected dir user.\n");
		return '';
	}

	my $pdirUser = new BU::PSA::Objects::ProtectedDirUser (
										$domainName,
										$pdir->getDBId(),
										$pdir->getField('path'),
										u2a($loginNode->getAttribute('name')),
										u2a($loginNode->getAttribute('password'))
									);

	unless ($pdirUser) {
		printf_stderr("Unable to create ProtectedDirUser object (%s/%s/%s).\n",
					$domainName, $pdir->getField('path'), $userName);
		return '';
	}

	if ($pdirUser->error()) {
		warn $pdirUser->getErrorMsg() . "\n";
		return '';
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		unless ($pdirUser->Exists()) {
			return '';
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {

		$pdirUser->Store();

		if ($pdirUser->error()) {
			warn $pdirUser->getErrorMsg() . "\n";
			return '';
		}
	}

	return 1;
}


return 1;

