#####################################################
package BU::PSA::Restore::SubDomain;
#####################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::XML::CommonFunc;
use BU::PSA::Objects::SysUser;
use BU::PSA::Restore::SiteApp;
use BU::PSA::Restore::ActionLog;


@ISA = qw(Exporter);
@EXPORT = qw(restoreSubDomain);

sub restoreSubDomain
{
	my ($baseNode, $domainId, $domainName, $hostingFtpNode) = @_;

	my $sdName = $baseNode->getAttribute('name');
	my $sdDisaplyName = u2a($baseNode->getAttribute('displayName'));

	printf_stdout("%s subdomain: '%s.%s'\n", $ProcessName, $sdName, $domainName);

	##############################################################
	### !!! ftp user node must be present in subdomains only if 
	### 	subdomain uses personal ftp user, not hosting ftp user
	my ($ftpNode) = $baseNode->xql('./user[@type="subdomain"]/login');
	if ($ftpNode) {
		$ftpLogin = $ftpNode->getAttribute('name');
		$ftpHomeDir = mapSysUserHomeDir(u2a($ftpNode->getAttribute('home')));
		$ftpNode->setAttribute('home', a2u($ftpHomeDir)) if $ftpHomeDir;
	} else {
		$ftpLogin = $hostingFtpNode->getAttribute('login');
		$ftpHomeDir = u2a($hostingFtpNode->getAttribute('home'));
	}


	my $sdId = undef;
	{ ### check subdomain existing
		my $query = dbDoQuery(sprintf('SELECT sd.id FROM domains d, subdomains sd WHERE d.name=%s AND d.id=sd.dom_id AND sd.name=%s', dbQuote($domainName), dbQuote($sdName)));
		unless ($query) {
			print_stderr("Unable to check subdomain existing.\n");
			return '';
		}
		($sdId) = $query->fetchrow_array();
		$subdomain_exists = $sdId && ($sdId+0 > 0);
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		if ($subdomain_exists) {
			$baseNode->setAttribute('exists', 'true');
			markCollision($baseNode);
			return 1;
		}

		$baseNode->setAttribute('destdir', $conf{HTTPD_VHOSTS_D} . "/$domainName/subdomains/$sdName");
        hashXMLNodeByCID($baseNode);
		if (my ($sn) = $baseNode->xql('./system_data')) {
			$sn->setAttribute('destdir', $conf{HTTPD_VHOSTS_D} . "/$domainName/subdomains/$sdName");
    	    hashXMLNodeByCID($sn);
		}

		if ($ftpNode) { ### subdomain has personal ftp user
			### if subdomain uses hosting ftp user then do not check collision
			my $query = dbDoQuery('SELECT id FROM sys_users WHERE login=' . dbQuote($ftpLogin));
			unless ($query) {
				print_stderr("Unable to check subdomain system user existing.\n");
				markCollision($baseNode);
				return '';
			}

			if (my ($su_id) = $query->fetchrow_array()) {
				#######################################
				### if such system user exists then:
				#######################################

				### check belonging to some domain
				my $query = dbDoQuery(sprintf('SELECT d.name FROM domains d, hosting h, sys_users su WHERE h.dom_id=d.id AND h.sys_user_id=su.id AND su.id=%s', dbQuote($domainName), $su_id));
				unless ($query) {
					print_stderr("Unable to check subdomain system user belonging to other domain.\n");
					markCollision($baseNode);
					return '';
				}
				if (my ($tmpDomainName) = $query->fetchrow_array()) {
					### if some domain uses such ftp user login then
					### it is in any case some other domain
					printf_stderr("The subdomain '%s' restoration is impossible:"
									."domain '%s' uses such ftp system user login for hosting.\n",
									$sdName, $tmpDomainName
								);
					markCollision($baseNode);
					return 1;
				}

				### check belonging to such subdomain
				my $query = dbDoQuery(sprintf('SELECT sd.id, sd.name FROM domains d, subdomains sd, sys_users su WHERE sd.dom_id=d.id AND d.name=%s AND sd.sys_user_id=su.id AND su.id=%s', dbQuote($domainName), dbQuote($su_id)));
				unless ($query) {
					print_stderr("Unable to check subdomain system user belonging to other subdomain.\n");
					markCollision($baseNode);
					return '';
				}

				if (my ($sd_id, $sdName) = $query->fetchrow_array()) {
					if ($sdName ne $baseNode-getAttribute('name')) {
						printf_stderr("The subdomain '%s' restoration is impossible:"
										."system user '%s' which is used for subdomain belongs to other subdomain.\n",
										$sdName, $domainName
									);
						markCollision($baseNode);
						return 1;
					}
				}
			}
		}

		### check such ftp user existing in system
		if (util_exec('usermng', 'exist', $ftpLogin)) {
			printf_stdout("'%s' user was found in the system\n", $ftpLogin);

			my @pw_info = getpwnam($ftpLogin);

			if (lc($pw_info[7]) ne lc($ftpHomeDir)) {
				printf_stderr("The subdomain restoration is impossible: "
							. "such ftp user already exists in the system and he/she in not "
							. "a valid plesk ftp user for subdomain.\n");

				markCollision($baseNode);
				return 1;
			}
			$ftpHomeDir = $pw_info[7];
			$ftpNode->setAttribute('home', a2u($ftpHomeDir)) if $ftpHomeDir;
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if ($baseNode->getAttribute('collision')) {
			printf_stderr("%s subdomain restoration is skipped.\n", $sdName);
			return 1;
		}

		##############################################
		### check for accordingly hosting services 
		###	(if some service is disabled for hosting
		###  it can't be enabled for subdomain)
		##############################################
		my $query = dbDoQuery(sprintf('SELECT h.* FROM domains d, hosting h WHERE d.name=%s AND d.id=h.dom_id', dbQuote($domainName)));
		unless ($query) {
			print_stderr("Unable to retrive hosting info from db.\n");
			return '';
		}
		unless ($h_hashref = $query->fetchrow_hashref()) {
			print_stderr("Unable to check subdomain services: hosting for domain is not exist,\n");
			return '';
		}
		my $hosting_sys_user_id = $$h_hashref{'sys_user_id'};

		@services_list = ('ssi', 'php', 'cgi', 'perl', 'python', 'coldfusion', 'asp', 'asp_dot_net');
		foreach $service (@services_list) {
			if ($$h_hashref{$service} ne 'true') {
				$baseNode->setAttribute($service, 'false');
			}
		}

		my $sys_user_id = undef;
		if ($ftpNode) {
			$su = new BU::PSA::Objects::SysUser(
						$ftpLogin,
						u2a($ftpNode->getAttribute('password')),
						$ftpNode->getAttribute('PW_TYPE'),
						$ftpHomeDir,
						$ftpNode->getAttribute('shell'),
						$ftpNode->getAttribute('quota'),
						);

			if ($su->error()) {
				print_stderr("Error during initialization SysUser class for subdomain: " . $su->getErrorMsg());
			}

			$su->Store();
			if ($su->error()) {
				print_stderr("Error during creating system user for subdomain: " . $su->getErrorMsg());
			}

			$sys_user_id = $su->getDBId();
		} else {
			$sys_user_id = $hosting_sys_user_id;
		}

		util_exec('subdomainmng', '--create', '--subdomain-name=' . $sdName,
				'--vhost-name=' . $domainName, '--user-name=' . $ftpLogin
		);

		dbDoQuery('INSERT INTO subdomains(dom_id, name, displayName, sys_user_id, sys_user_type, ssi, php, cgi, perl, python, coldfusion, asp, asp_dot_net)'
				. 'VALUES(' .
					dbQuote($domainId) . ',' .
					dbQuote($baseNode->getAttribute('name')) . ',' .
					dbQuote(u2a($baseNode->getAttribute('displayName'))) . ',' .
					dbQuote($sys_user_id) . ',' .
					dbQuote($baseNode->getAttribute('sys_user_type')) . ',' .
					dbQuote($baseNode->getAttribute('ssi')) . ',' .
					dbQuote($baseNode->getAttribute('php')) . ',' .
					dbQuote($baseNode->getAttribute('cgi')) . ',' .
					dbQuote($baseNode->getAttribute('perl')) . ',' .
					dbQuote($baseNode->getAttribute('python')) . ',' .
					dbQuote($baseNode->getAttribute('coldfusion')) . ',' .
					dbQuote($baseNode->getAttribute('asp')) . ',' .
					dbQuote($baseNode->getAttribute('asp_dot_net')) . ')'
				);
	}
	$sdId = dbGetInsertId();

	unless ($subdomain_exists) {
		restoreActionLog($baseNode, $sdId, 'subdomain');
	}

	my $domainNode = $baseNode->getParentNode()->getParentNode()->getParentNode();
	if ($domainNode->getNodeName() ne 'domain') {
		print_stderr("FATAL error: subdomain parent node is not 'domain', it is '".$domainNode->getNodeName()."'\n");
		die "\n";
	}
	# restore SiteApps
	foreach my $siteAppNode ($baseNode->xql('./site_apps/*')) {
		if (($flags{RESTORE_MODE} eq 'CHECK') && ($baseNode->getAttribute('content_restore', 'false'))) {
			markCollision($siteAppNode);
		} else {
			restoreSiteApp($siteAppNode, $domainNode, $domainId, $domainName, $sdName, $sdId);
		}
	}
	if (($flags{RESTORE_MODE} eq 'CHECK') && ($baseNode->getAttribute('content_restore', 'false'))) {
		print_stderr("Content and all site applications for subdomain '$sdName' (doman '$domainName') will not be restored because site application collision detected\n");
	}

	if ($flags{RESTORE_MODE} eq 'RESTORE') {
		### default permissions setting (it is need for RedHat 9.0)
		if (($const{OSid} eq 'linux') && ($const{linux_distr} eq 'redhat') && (ver2int($const{OS_VERSION}) >= 900000000)) {
			my $uid = $ftpNode->getAttribute('uid');
			if ($ftpNode && ($uid ne '') && ($uid+0 != getpwnam($ftpLogin)+0)) {
				system("find $ftpHomeDir \\( -user $uid -or -nouser \\) -exec chown $ftpLogin {} \; > /dev/null 2>&1");
			}
		}
	}

	return 1;
}
