#####################################################
package BU::PSA::Restore::WebApps;
#####################################################

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::DSTomcat;
use BU::XML::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(restoreDSTomcat restoreWebApp);


sub restoreDSTomcat
{
	my ($base_node, $dom_id, $domainName, $www_prefix) = @_;

	unless ($conf{'TOMCAT_IS_INSTALLED'}) {
		return 1;
	}

	my $coyote_connector_port = getMiscParam('coyote_connector_port');
	my $warp_connector_port   = getMiscParam('warp_connector_port'  );

	$coyote_connector_port = ($coyote_connector_port) ? $coyote_connector_port : 9080;
	$warp_connector_port   = ($warp_connector_port  ) ? $warp_connector_port   : 9008;

	if ($PRETTYPRINT) {
		print_sl("Tomcat domain service");
	} else {
		printf_stdout("%s Tomcat domain service.\n", $ProcessName, $domainName);
	}

	my $dsTomcat = new BU::PSA::Objects::DSTomcat($dom_id, $domainName, 0, $www_prefix, $coyote_connector_port, $warp_connector_port);
	if ($dsTomcat->error()) {
		printf_stderr("Error DSTomcat object creating: %s", $dsTomcat->getErrorMsg());
		return '';
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
#		return 1 unless $dsTomcat->Exists();
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		my $status = $base_node->getAttribute('status');
		$status = $status ^ ($status & $const{'STATUS_WITH_PARENT'});

		$dsTomcat->setField('type', $base_node->getAttribute('type'));
		$dsTomcat->setField('status', $status);
		$dsTomcat->Store();
		if ($dsTomcat->error()) {
			printf_stderr("Tomcat restoring error: %s\n", $dsTomcat->getErrorMsg());
			return '';
		}

		####################################################################
		#	Restore service parameters: manager_login and manager_password
		####################################################################
		$dsTomcat->update();
		$dsTomcat->Store();
	}

	foreach my $node ($base_node->xql('./web_app')) {
		restoreWebApp($node, $dsTomcat->dbExists() ? $dsTomcat->getDBId() : 0, $domainName);
	}
}


sub restoreWebApp
{
	my ($node, $domain_service_id, $domainName) = @_;

	my $path = $conf{CATALINA_HOME} . "/psa-wars/$domainName";
	my $name = u2a($node->getAttribute('name'));

	if ($PRETTYPRINT) {
		print_sl("Tomcat web application");
	} else {
		printf_stdout "%s Tomcat web application for %s\n", $ProcessName, $domainName;
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
        	hashXMLNodeByCID($node);
			$node->setAttribute('destfile', "$path/$name.war");

			if ($domain_service_id) {
				my $query = dbDoQuery("SELECT id FROM WebApps WHERE domain_service_id='$domain_service_id' AND name=". dbQuote($name));
				if (my ($tmp) = $query->fetchrow_array()) {
					$node->setAttribute('exists', 'true');
				}
			}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if ($node->getAttribute('exists') eq 'true') {
			return 1;
		}

		if ( -f "$path/$name.war" ) {
			my $status = $node->getAttribute('status');

			dbDoQuery(	"REPLACE INTO WebApps(name, status, domain_service_id) ".
						"VALUES(". dbQuote($name) .", '$status', '$domain_service_id')");

			if ($status == 0) {
				util_exec('tomcatmng', '--start-webapp', "--vhost-name=$domainName", "--webapp-name=$name");
			}
		}
	}

	return 1;
}

return 1;

