package BU::PSA::Restore::WebUser;

require 5.004;
require Exporter;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::PSA::Objects::WebUser;
use BU::XML::CommonFunc;
use BU::PSA::Restore::ActionLog;


@ISA = qw(Exporter);
@EXPORT = qw(restoreWebUser);

sub restoreWebUser($ $ $)
{
	my ($domainId, $domainName, $webUserNode) = @_;

	my $loginNode = '';

	unless (($loginNode) = $webUserNode->xql('./login')) {
		print_stderr("Unable to define login name for web user: XML tag does not exist.\n");
		return '';
	}

	my $webUserLogin = u2a($loginNode->getAttribute('name'));
	my $webUserHomeDir = mapSysUserHomeDir(u2a($loginNode->getAttribute('home')));
	$loginNode->setAttribute('home', a2u($webUserHomeDir));

	printf_stdout("%s web user '%s'\n", $ProcessName, $webUserLogin);

	my $isExist = '';
	my $webUser = undef;
	if ($domainId) {
		$webUser = new BU::PSA::Objects::WebUser(
									$domainId,
									$webUserLogin,
									u2a($loginNode->getAttribute('password')),
									$loginNode->getAttribute('PW_TYPE'),
									$webUserHomeDir,
									u2a($loginNode->getAttribute('shell')),
									$loginNode->getAttribute('quota'),
								);

		unless ($webUser) {
			printf_stderr("Unable to create %s object for '%s' web user\n",  "BU::PSA::Webuser", $webUserLogin);
			return '';
		}

		if ($webUser->error()) {
			warn $webUser->getErrotMsg();
			return '';
		}
		$isExist = $webUser->dbExists();
		if ($isExist) {
			my $query = dbDoQuery('SELECT login FROM sys_users WHERE lower(login)=lower(' . dbQuote($webUserLogin) . ')');
			if (my ($tmp) = $query->fetchrow_array()) {
				####################################
				### IGNORE CASE FOR SYS USER LOGIN
				$webUserLogin = $tmp;
				$loginNode->setAttribute('name', a2u($webUserLogin));
				$webUser->{sysUser}->setField('login', $webUserLogin);
				####################################
			}
		}
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		$webUserNode->setAttribute('destdir', $conf{'HTTPD_VHOSTS_D'} . "/$domainName/web_users");
        hashXMLNodeByCID($webUserNode);

		if (my ($crontabNode) = $webUserNode->xql('./crontab')) {
        	hashXMLNodeByCID($crontabNode);
		}

		if ($isExist) {
			$webUserNode->setAttribute('exists', 'true');
			print_stderr("'$webUserLogin' user was found in the system.\n");
		}

		{ ### check for web user collision
			my $query = dbDoQuery('SELECT name FROM web_users wu, domains d, sys_users su WHERE wu.dom_id=d.id'
									. ' and wu.sys_user_id=su.id and su.login=' . dbQuote($webUserLogin));

			my ($webUserDomainName) = $query->fetchrow_array if $query;

			unless ($query || $webUserDomainName) {
				print_stderr("Unable to check whether a web user related to the domain %s exists in the PSA system.\n", $domainName);
			} elsif ($webUserDomainName && ($domainName ne $webUserDomainName)) {
				printf_stderr("'%s' web user restoration is impossible: such user already ".
					 "exists in PSA but he/she is related to the domain %s\n",
					 $webUserLogin, $domainName);
				markCollision($webUserNode);
				return '';
			}
		}

		if (util_exec('usermng', ('exist', $webUserLogin))) {
			my @pw_info = getpwnam($webUserLogin);
			if ($pw_info[7] ne $webUserHomeDir) {
				printf_stderr("'%s' web user restoration is impossible: user under such name already exists in the system and he is not valid PSA web user.\n",
							 $webUserLogin);
				markCollision($webUserNode);
				return '';
			}
		}

		$conf{'DOMAIN_WEBUSERS_USAGE'}++ unless $isExist;

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {

		if ($webUserNode->getAttribute('collision')) {
			printf_stderr("'%s' web user restoration is skipped.\n", $webUserLogin);
			return '';
		}
	
		if (my ($servicesNode) = $webUserNode->xql('./services')) {

			my $serviceNodesList = $servicesNode->getAttributes();

			for (my $i=0; $i<$serviceNodesList->getLength; $i++) {
				my $serviceNode = $serviceNodesList->item($i);
				$webUser->setField($serviceNode->getNodeName(), u2a($serviceNode->getNodeValue()));
			}
		}

		if ($webUser->error()) {
			warn $webUser->getErrorMsg();
			return '';
		}

		$webUser->Store();

		if ($webUser->error()) {
			printf_stderr("Unable to save web user with login '%s': %s\n", $webUserLogin, $webUser->getErrorMsg());
			return '';
		}

		unless ($isExist) {
			restoreActionLog($webUserNode, $webUser->getDBId(), 'webuser');
		} else {
			$webUser->logChanges($webUser->getDBId(), 'webuser_update');
		}

		if ($loginNode->getAttribute('PW_TYPE') eq 'crypt') {
			my $password = u2a($loginNode->getAttribute('password'));
			setPasswd($webUserLogin, $password) if $password;
		}
	}

	return 1;
}


return 1;

