###################################################################
package BU::XML::CommonFunc;
###################################################################

require 5.004;
require Exporter;
use Carp;
use XML::DOM;
use BU::DB qw(dbQuote);
use BU::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(markCollision XMLAttrs2SQL XMLNodesList2SQL createNodeFromHash);

# This function creates a xml Node with name $nodeName and set attributes for one gived from hash %hash
sub createNodeFromHash {
	my ($doc, $nodeName, $hash) = @_;

	my $node = $doc->createElement($nodeName);

	while (my ($field, $value) = each %$hash) {
		$node->setAttribute($field, a2u($value));
	}

	return $node;
}

sub markCollision($)
{
	my ($objectNode) = @_;

	$objectNode->setAttribute('collision', 'true'); 
	foreach my $subNode ($objectNode->xql('.//*[@src]')) {
		$subNode->setAttribute('collision', 'true');
	}
}


sub XMLAttrs2SQL($ $; $)
{
	my ($format_str, $attrsNodesList, $decode_base64) = @_;

	my $fields_str = '';
	my $values_str = '';
	my $coma = '';
	for (my $i=0; $i<$attrsNodesList->getLength; $i++) {
		my $node = $attrsNodesList->item($i);
		$fields_str .= $coma . $node->getNodeName();
		if ($decode_base64) {
			$values_str .= $coma . dbQuote(u2a($node->getNodeValue()));
		} else {
			$values_str .= $coma . dbQuote($node->getNodeValue());
		}
		$coma = ', ';
	}

	return sprintf($format_str, $fields_str, $values_str);
}


sub XMLNodesList2SQL($ $ $)
{
	my ($format_str, $nodesList, $hash_params) = @_;
	# hash_params: decode_base64, map_fields

	my $map_fields = exists ($hash_params->{map_fields}) ? $hash_params->{map_fields} : undef;

	my $fields_str = '';
	my $values_str = '';
	my $coma = '';
	for (my $i=0; $i<$nodesList->getLength; $i++) {
		my $node = $nodesList->item($i);
		$fields_str .= $coma . ($map_fields ? $map_fields->{$node->getNodeName()} : $node->getNodeName());

		my $text = '';
		my $text_node = $node->getFirstChild();

		if ($text_node) {
			$text = ($hash_params->{decode_base64}) ? u2a($text_node->toString) : $text_node->toString;
		}

		$values_str .= $coma . dbQuote($text);
		$coma = ', ';
	}

	return sprintf($format_str, $fields_str, $values_str);
}
